#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

from argparse import ArgumentParser, Namespace

from idb.cli.commands.base import TargetCommand
from idb.common.types import IdbClient


class ApproveCommand(TargetCommand):
    @property
    def description(self) -> str:
        return "Approve permissions for an app"

    @property
    def name(self) -> str:
        return "approve"

    def add_parser_arguments(self, parser: ArgumentParser) -> None:
        parser.add_argument("bundle_id", help="App's bundle id", type=str)
        parser.add_argument(
            "permissions",
            nargs="+",
            help="Permissions to approve",
            choices=["photos", "camera", "contacts"],
        )
        super().add_parser_arguments(parser)

    async def run_with_client(self, args: Namespace, client: IdbClient) -> None:
        await client.approve(
            bundle_id=args.bundle_id, permissions=set(args.permissions)
        )



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

from argparse import ArgumentParser, Namespace
from typing import List

from idb.cli.commands.base import TargetCommand
from idb.common.signal import signal_handler_event
from idb.common.types import IdbClient


class RecordVideoCommand(TargetCommand):
    @property
    def description(self) -> str:
        return "Record the target's screen to a mp4 video file"

    @property
    def name(self) -> str:
        return "video"

    @property
    def aliases(self) -> List[str]:
        return ["record-video"]

    def add_parser_arguments(self, parser: ArgumentParser) -> None:
        parser.add_argument("output_file", help="mp4 file to output the video to")
        super().add_parser_arguments(parser)

    async def run_with_client(self, args: Namespace, client: IdbClient) -> None:
        await client.record_video(
            stop=signal_handler_event("video"), output_file=args.output_file
        )



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.


import asyncio
import json
import logging
import socket
from contextlib import closing
from typing import Optional, Tuple

import aiofiles


DEFAULT_NETWORK_TIMEOUT: int = 3


def is_port_open(host: str, port: int) -> bool:
    with closing(socket.socket(socket.AF_INET6, socket.SOCK_STREAM)) as sock:
        sock.settimeout(DEFAULT_NETWORK_TIMEOUT)
        return sock.connect_ex((host, port)) == 0


async def gen_listening_ports_from_fd(
    process: asyncio.subprocess.Process,
    read_fd: int,
    timeout: Optional[int] = None,
    logger: Optional[logging.Logger] = None,
) -> Tuple[int, Optional[int]]:
    if logger is None:
        logger = logging.getLogger("reply-fd")
    wait = asyncio.ensure_future(process.wait())
    ports = asyncio.ensure_future(_read_from_fd(read_fd, logger))
    done, pending = await asyncio.wait(
        [wait, ports], return_when=asyncio.FIRST_COMPLETED
    )
    for fut in pending:
        fut.cancel()

    if ports not in done:
        raise Exception(
            f"Process exited with return code {process.returncode} before "
            f"responding with port"
        )
    return await ports


async def _read_from_fd(
    read_fd: int, logger: logging.Logger
) -> Tuple[int, Optional[int]]:
    async with aiofiles.open(read_fd, "r") as f:
        logger.info(f"Opened read_fd: {read_fd}")
        data = await f.readline()
        logger.info(f"Opened with data: {data} on read_fd: {read_fd}")
        return _get_ports(data)


def _get_ports(data: str) -> Tuple[int, Optional[int]]:
    all_ports = json.loads(data)
    return all_ports["grpc_port"]



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

import asyncio
import json
import logging
import os
from asyncio import StreamReader
from typing import List

from idb.client.pid_saver import PidSaver
from idb.common.constants import IDB_LOCAL_TARGETS_FILE, IDB_LOGS_PATH


class CompanionSpawnerException(Exception):
    pass


class CompanionSpawner:
    def __init__(self, companion_path: str, logger: logging.Logger) -> None:
        self.companion_path = companion_path
        self.logger = logger
        self.pid_saver = PidSaver(logger=self.logger)

    async def _read_stream(self, stream: StreamReader) -> int:
        port = 0
        while True:
            line = await stream.readline()
            logging.debug(f"read line from companion : {line}")
            if line:
                update = json.loads(line.decode())
                if update:
                    logging.debug(f"got update from companion {update}")
                    port = update["grpc_port"]
                    break
            else:
                break
        return port

    def _log_file_path(self, target_udid: str) -> str:
        os.makedirs(name=IDB_LOGS_PATH, exist_ok=True)
        return IDB_LOGS_PATH + "/" + target_udid

    async def spawn_companion(self, target_udid: str) -> int:
        if not self.companion_path:
            raise CompanionSpawnerException(
                f"couldn't instantiate a companion for {target_udid} because\
                 the companion_path is not available"
            )
        cmd: List[str] = [
            self.companion_path,
            "--udid",
            target_udid,
            "--grpc-port",
            "0",
        ]

        with open(self._log_file_path(target_udid), "a") as log_file:
            process = await asyncio.create_subprocess_exec(
                *cmd,
                stdout=asyncio.subprocess.PIPE,
                stdin=asyncio.subprocess.PIPE,
                stderr=log_file,
            )
            self.pid_saver.save_companion_pid(pid=process.pid)
            logging.debug(f"started companion at process id {process.pid}")
            if process.stdout:
                port = await self._read_stream(process.stdout)
                if not port:
                    raise CompanionSpawnerException("failed to spawn companion")
                return port
            raise CompanionSpawnerException("process has no stdout")

    def _is_notifier_running(self) -> bool:
        return self.pid_saver.get_notifier_pid() > 0

    async def spawn_notifier(self) -> None:
        if not self._is_notifier_running():
            if not self.companion_path:
                raise CompanionSpawnerException(
                    f"couldn't instantiate a notifier because\
                     the companion_path is not available"
                )
            cmd: List[str] = [self.companion_path, "--notify", IDB_LOCAL_TARGETS_FILE]

            with open(self._log_file_path("notifier"), "a") as log_file:
                process = await asyncio.create_subprocess_exec(
                    *cmd, stdout=log_file, stderr=log_file
                )
                self.pid_saver.save_notifier_pid(pid=process.pid)
                logging.debug(f"started notifier at process id {process.pid}")



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

from typing import List, Tuple, TypeVar

from idb.common.types import (
    HIDButton,
    HIDButtonType,
    HIDDelay,
    HIDDirection,
    HIDEvent,
    HIDKey,
    HIDPress,
    HIDPressAction,
    HIDSwipe,
    HIDTouch,
    Point,
)
from idb.grpc.idb_pb2 import HIDEvent as GrpcHIDEvent, Point as GrpcPoint


GrpcHIDButton = GrpcHIDEvent.HIDButton
GrpcHIDDelay = GrpcHIDEvent.HIDDelay
GrpcHIDKey = GrpcHIDEvent.HIDKey
GrpcHIDPress = GrpcHIDEvent.HIDPress
GrpcHIDPressAction = GrpcHIDEvent.HIDPressAction
GrpcHIDSwipe = GrpcHIDEvent.HIDSwipe
GrpcHIDTouch = GrpcHIDEvent.HIDTouch
GrpcHIDButtonType = GrpcHIDEvent.HIDButtonType
GrpcHIDDirection = GrpcHIDEvent.HIDDirection
_A = TypeVar("_A")
_B = TypeVar("_B")


BUTTON_TYPE_PAIRS: "List[Tuple[HIDButtonType, GrpcHIDButtonType]]" = [
    (HIDButtonType.APPLE_PAY, GrpcHIDEvent.APPLE_PAY),
    (HIDButtonType.HOME, GrpcHIDEvent.HOME),
    (HIDButtonType.LOCK, GrpcHIDEvent.LOCK),
    (HIDButtonType.SIDE_BUTTON, GrpcHIDEvent.SIDE_BUTTON),
    (HIDButtonType.SIRI, GrpcHIDEvent.SIRI),
]

DIRECTION_PAIRS: "List[Tuple[HIDDirection, GrpcHIDDirection]]" = [
    (HIDDirection.DOWN, GrpcHIDEvent.DOWN),
    (HIDDirection.UP, GrpcHIDEvent.UP),
]


def _tanslation_from_pairs(pairs: List[Tuple[_A, _B]], item: _A) -> _B:
    pair_map = {py: grpc for (py, grpc) in pairs}
    return pair_map[item]


def button_type_to_grpc(button_type: HIDButtonType) -> GrpcHIDButtonType:
    return _tanslation_from_pairs(BUTTON_TYPE_PAIRS, button_type)


def direction_to_grpc(direction: HIDDirection) -> GrpcHIDDirection:
    return _tanslation_from_pairs(DIRECTION_PAIRS, direction)


def point_to_grpc(point: Point) -> GrpcPoint:
    return GrpcPoint(x=point.x, y=point.y)


def touch_to_grpc(touch: HIDTouch) -> GrpcHIDTouch:
    return GrpcHIDTouch(point=point_to_grpc(touch.point))


def button_to_grpc(button: HIDButton) -> GrpcHIDButton:
    return GrpcHIDButton(button=button_type_to_grpc(button.button))


def key_to_grpc(key: HIDKey) -> GrpcHIDKey:
    return GrpcHIDKey(keycode=key.keycode)


def press_action_to_grpc(action: HIDPressAction) -> GrpcHIDPressAction:
    if isinstance(action, HIDTouch):
        return GrpcHIDPressAction(touch=touch_to_grpc(action))
    elif isinstance(action, HIDButton):
        return GrpcHIDPressAction(button=button_to_grpc(action))
    elif isinstance(action, HIDKey):
        return GrpcHIDPressAction(key=key_to_grpc(action))
    else:
        raise Exception(f"Invalid press action {action}")


def press_to_grpc(press: HIDPress) -> GrpcHIDPress:
    return GrpcHIDPress(
        action=press_action_to_grpc(press.action),
        direction=direction_to_grpc(press.direction),
    )


def swipe_to_grpc(swipe: HIDSwipe) -> GrpcHIDSwipe:
    return GrpcHIDSwipe(
        start=point_to_grpc(swipe.start),
        end=point_to_grpc(swipe.end),
        delta=swipe.delta,
    )


def delay_to_grpc(delay: HIDDelay) -> GrpcHIDDelay:
    return GrpcHIDDelay(duration=delay.duration)


def event_to_grpc(event: HIDEvent) -> GrpcHIDEvent:
    if isinstance(event, HIDPress):
        return GrpcHIDEvent(press=press_to_grpc(event))
    elif isinstance(event, HIDSwipe):
        return GrpcHIDEvent(swipe=swipe_to_grpc(event))
    elif isinstance(event, HIDDelay):
        return GrpcHIDEvent(delay=delay_to_grpc(event))
    else:
        raise Exception(f"Invalid event {event}")



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

import json
from unittest import mock

from idb.common.companion_spawner import CompanionSpawner
from idb.common.constants import IDB_LOCAL_TARGETS_FILE
from idb.utils.testing import AsyncMock, TestCase, ignoreTaskLeaks


@ignoreTaskLeaks
class CompanionSpawnerTest(TestCase):
    async def test_spawn_companion(self) -> None:
        spawner = CompanionSpawner("idb_path", logger=mock.Mock())
        spawner._log_file_path = mock.Mock()
        spawner.pid_saver = mock.Mock()
        udid = "someUdid"
        with mock.patch(
            "idb.common.companion_spawner.asyncio.create_subprocess_exec",
            new=AsyncMock(),
        ) as exec_mock, mock.patch("idb.common.companion_spawner.open"):
            process_mock = mock.Mock()
            process_mock.stdout.readline = AsyncMock(
                return_value=json.dumps(
                    {"hostname": "myHost", "grpc_port": 1234}
                ).encode("utf-8")
            )
            exec_mock.return_value = process_mock
            port = await spawner.spawn_companion(udid)
            exec_mock.assert_called_once_with(
                "idb_path",
                "--udid",
                "someUdid",
                "--grpc-port",
                "0",
                stdout=mock.ANY,
                stdin=mock.ANY,
                stderr=mock.ANY,
            )
            self.assertEqual(port, 1234)

    async def test_spawn_notifier(self) -> None:
        spawner = CompanionSpawner("idb_path", logger=mock.Mock())
        spawner._log_file_path = mock.Mock()
        spawner._is_notifier_running = mock.Mock(return_value=False)
        spawner.pid_saver = mock.Mock()
        with mock.patch(
            "idb.common.companion_spawner.asyncio.create_subprocess_exec",
            new=AsyncMock(),
        ) as exec_mock, mock.patch("idb.common.companion_spawner.open"):
            process_mock = mock.Mock()
            exec_mock.return_value = process_mock
            await spawner.spawn_notifier()
            exec_mock.assert_called_once_with(
                "idb_path",
                "--notify",
                IDB_LOCAL_TARGETS_FILE,
                stderr=mock.ANY,
                stdout=mock.ANY,
            )



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

from argparse import Namespace

from idb.cli.commands.base import TargetCommand
from idb.common.types import IdbClient


class DescribeCommand(TargetCommand):
    @property
    def description(self) -> str:
        return "Describes the Target"

    @property
    def name(self) -> str:
        return "describe"

    async def run_with_client(self, args: Namespace, client: IdbClient) -> None:
        description = await client.describe()
        print(description)



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

import json

from idb.common.format import (
    installed_app_info_from_json,
    installed_test_info_from_json,
    json_data_companions,
    json_format_installed_app_info,
    json_format_installed_test_info,
    json_format_target_info,
    json_format_test_info,
    json_to_companion_info,
    target_description_from_json,
    test_info_from_json,
)
from idb.common.types import (
    AppProcessState,
    CompanionInfo,
    InstalledAppInfo,
    InstalledTestInfo,
    TargetDescription,
    TestActivity,
    TestRunFailureInfo,
    TestRunInfo,
)
from idb.utils.testing import TestCase


TEST_RUN_FAILURE_INFO_FIXTURE = TestRunFailureInfo(
    message="FailedMsg", file="test.py", line=7
)
TEST_ACTIVITY_FIXTURE = TestActivity(title="ActivityTitle", duration=8, uuid="MyUdid")
TEST_RUN_INFO_FIXTURE = TestRunInfo(
    bundle_name="MyBundleName",
    class_name="MyClassName",
    method_name="MyMethodName",
    logs=["logA", "logB"],
    duration=12.34,
    passed=True,
    failure_info=None,
    activityLogs=[],
    crashed=False,
)
INSTALLED_APP_INFO_FIXTURE = InstalledAppInfo(
    bundle_id="MyBundleId",
    name="MyName",
    architectures={"ArchA", "ArchB"},
    install_type="System",
    process_state=AppProcessState.RUNNING,
    debuggable=True,
)
COMPANION_INFO_FIXTURE = CompanionInfo(
    udid="MyUdid", host="ThisMac", port=1234, is_local=False
)
TARGET_DESCRIPTION_FIXTURE = TargetDescription(
    udid="MyUdid",
    name="MyName",
    state="Started?",
    target_type="iOS",
    os_version="1",
    architecture="arm89",
    companion_info=None,
    screen_dimensions=None,
)
INSTALLED_TEST_INFO = InstalledTestInfo(
    bundle_id="MyBundleID", name="MyName", architectures={"ArchA", "ArchB"}
)


class FormattingTests(TestCase):
    def test_json_to_companion_info(self) -> None:
        self.assertEqual(
            [COMPANION_INFO_FIXTURE],
            json_to_companion_info(json_data_companions([COMPANION_INFO_FIXTURE])),
        )

    def test_test_info_no_optional_fields(self) -> None:
        self.assertEqual(
            TEST_RUN_INFO_FIXTURE,
            test_info_from_json(json_format_test_info(TEST_RUN_INFO_FIXTURE)),
        )

    def test_test_info_all_optional_fields(self) -> None:
        info = TEST_RUN_INFO_FIXTURE._replace(
            failure_info=TEST_RUN_FAILURE_INFO_FIXTURE,
            activityLogs=[TEST_ACTIVITY_FIXTURE],
        )
        self.assertEqual(info, test_info_from_json(json_format_test_info(info)))

    def test_installed_app_info(self) -> None:
        self.assertEqual(
            INSTALLED_APP_INFO_FIXTURE,
            installed_app_info_from_json(
                json_format_installed_app_info(INSTALLED_APP_INFO_FIXTURE)
            ),
        )

    def test_target_description_no_optional_fields(self) -> None:
        self.assertEqual(
            TARGET_DESCRIPTION_FIXTURE,
            target_description_from_json(
                json_format_target_info(TARGET_DESCRIPTION_FIXTURE)
            ),
        )

    def test_target_description_all_optional_fields(self) -> None:
        target = TARGET_DESCRIPTION_FIXTURE._replace(
            companion_info=COMPANION_INFO_FIXTURE
        )
        self.assertEqual(
            target, target_description_from_json(json_format_target_info(target))
        )

    def test_installed_test_info(self) -> None:
        self.assertEqual(
            INSTALLED_TEST_INFO,
            installed_test_info_from_json(
                json_format_installed_test_info(INSTALLED_TEST_INFO)
            ),
        )



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

from idb.common.types import Address, ConnectionDestination
from idb.grpc.idb_pb2 import (
    ConnectionAddress as GrpcConnectionAddress,
    ConnectionDestination as GrpcConnectionDestination,
)


def destination_to_grpc(
    destination: ConnectionDestination
) -> GrpcConnectionDestination:
    if isinstance(destination, str):
        return GrpcConnectionDestination(target_udid=destination)
    elif isinstance(destination, Address):
        return GrpcConnectionDestination(
            address=GrpcConnectionAddress(host=destination.host, port=destination.port)
        )


def destination_to_py(destination: GrpcConnectionDestination) -> ConnectionDestination:
    if destination.HasField("address"):
        return Address(host=destination.address.host, port=destination.address.port)
    else:
        return destination.target_udid



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

import asyncio
import functools
import inspect
import logging
import os
import urllib.parse
from io import StringIO
from pathlib import Path
from sys import platform
from typing import (
    Any,
    AsyncIterable,
    AsyncIterator,
    Dict,
    Iterable,
    List,
    Optional,
    Set,
    Tuple,
)

from grpclib.client import Channel
from grpclib.exceptions import GRPCError, ProtocolError, StreamTerminatedError
from idb.client.pid_saver import PidSaver
from idb.common.companion_spawner import CompanionSpawner
from idb.common.constants import TESTS_POLL_INTERVAL
from idb.common.direct_companion_manager import DirectCompanionManager
from idb.common.gzip import drain_gzip_decompress
from idb.common.hid import (
    button_press_to_events,
    iterator_to_async_iterator,
    key_press_to_events,
    swipe_to_events,
    tap_to_events,
    text_to_events,
)
from idb.common.install import (
    Bundle,
    Destination,
    generate_binary_chunks,
    generate_io_chunks,
    generate_requests,
)
from idb.common.instruments import (
    drain_until_running,
    instruments_generate_bytes,
    translate_instruments_timings,
)
from idb.common.launch import drain_launch_stream, end_launch_stream
from idb.common.local_targets_manager import LocalTargetsManager
from idb.common.logging import log_call
from idb.common.stream import stream_map
from idb.common.tar import create_tar, drain_untar, generate_tar
from idb.common.types import (
    AccessibilityInfo,
    Address,
    AppProcessState,
    CompanionInfo,
    ConnectionDestination,
    CrashLog,
    CrashLogInfo,
    CrashLogQuery,
    FileEntryInfo,
    HIDButtonType,
    HIDEvent,
    IdbClient,
    IdbException,
    InstalledAppInfo,
    InstalledArtifact,
    InstalledTestInfo,
    InstrumentsTimings,
    LoggingMetadata,
    TargetDescription,
    TestRunInfo,
)
from idb.common.video import generate_video_bytes
from idb.common.xctest import make_request, make_results, write_result_bundle
from idb.grpc.idb_grpc import CompanionServiceStub
from idb.grpc.idb_pb2 import (
    AccessibilityInfoRequest,
    AddMediaRequest,
    ApproveRequest,
    ClearKeychainRequest,
    ConnectRequest,
    ContactsUpdateRequest,
    CrashShowRequest,
    DebugServerRequest,
    DebugServerResponse,
    FocusRequest,
    InstallRequest,
    InstrumentsRunRequest,
    LaunchRequest,
    ListAppsRequest,
    Location,
    LogRequest,
    LsRequest,
    MkdirRequest,
    MvRequest,
    OpenUrlRequest,
    Payload,
    Point,
    PullRequest,
    PullResponse,
    PushRequest,
    RecordRequest,
    RmRequest,
    ScreenshotRequest,
    SetLocationRequest,
    TargetDescriptionRequest,
    TerminateRequest,
    UninstallRequest,
    XctestListBundlesRequest,
    XctestListTestsRequest,
)
from idb.grpc.stream import (
    cancel_wrapper,
    drain_to_stream,
    generate_bytes,
    stop_wrapper,
)
from idb.ipc.mapping.crash import (
    _to_crash_log,
    _to_crash_log_info_list,
    _to_crash_log_query_proto,
)
from idb.ipc.mapping.destination import destination_to_grpc
from idb.ipc.mapping.hid import event_to_grpc
from idb.ipc.mapping.target import target_to_py
from idb.utils.contextlib import asynccontextmanager
from idb.utils.typing import none_throws


APPROVE_MAP: Dict[str, Any] = {
    "photos": ApproveRequest.PHOTOS,
    "camera": ApproveRequest.CAMERA,
    "contacts": ApproveRequest.CONTACTS,
}


CLIENT_METADATA: LoggingMetadata = {"component": "client", "rpc_protocol": "grpc"}


def log_and_handle_exceptions(func):  # pyre-ignore
    @functools.wraps(func)
    @log_call(name=func.__name__, metadata=CLIENT_METADATA)
    async def func_wrapper(*args: Any, **kwargs: Any) -> Any:  # pyre-ignore
        try:
            return await func(*args, **kwargs)
        except GRPCError as e:
            raise IdbException(e.message) from e  # noqa B306
        except (ProtocolError, StreamTerminatedError) as e:
            raise IdbException(e.args) from e

    @functools.wraps(func)
    @log_call(name=func.__name__, metadata=CLIENT_METADATA)
    async def func_wrapper_gen(*args: Any, **kwargs: Any) -> Any:  # pyre-ignore
        try:
            async for item in func(*args, **kwargs):
                yield item
        except GRPCError as e:
            raise IdbException(e.message) from e  # noqa B306
        except (ProtocolError, StreamTerminatedError) as e:
            raise IdbException(e.args) from e

    if inspect.isasyncgenfunction(func):
        return func_wrapper_gen
    else:
        return func_wrapper


class GrpcClient(IdbClient):
    def __init__(
        self, target_udid: Optional[str], logger: Optional[logging.Logger] = None
    ) -> None:
        self.logger: logging.Logger = (
            logger if logger else logging.getLogger("idb_grpc_client")
        )
        self.target_udid = target_udid
        self.direct_companion_manager = DirectCompanionManager(logger=self.logger)
        self.local_targets_manager = LocalTargetsManager(logger=self.logger)
        self.companion_info: Optional[CompanionInfo] = None

    async def spawn_notifier(self) -> None:
        if platform == "darwin":
            companion_spawner = CompanionSpawner(
                companion_path="idb_companion", logger=self.logger
            )
            await companion_spawner.spawn_notifier()

    @asynccontextmanager
    async def get_stub(self) -> CompanionServiceStub:
        await self.spawn_notifier()
        channel: Optional[Channel] = None
        try:
            try:
                self.companion_info = self.direct_companion_manager.get_companion_info(
                    target_udid=self.target_udid
                )
            except IdbException as e:
                # will try to spawn a companion if on mac.
                companion_info = await self.spawn_companion(
                    target_udid=none_throws(self.target_udid)
                )
                if companion_info:
                    self.companion_info = companion_info
                else:
                    raise e
            self.logger.info(f"using companion {self.companion_info}")
            channel = Channel(
                self.companion_info.host,
                self.companion_info.port,
                loop=asyncio.get_event_loop(),
            )
            yield CompanionServiceStub(channel=channel)
        finally:
            if channel:
                channel.close()

    async def spawn_companion(self, target_udid: str) -> Optional[CompanionInfo]:
        if self.local_targets_manager.is_local_target_available(
            target_udid=target_udid
        ):
            companion_spawner = CompanionSpawner(
                companion_path="idb_companion", logger=self.logger
            )
            self.logger.info(f"will attempt to spawn a companion for {target_udid}")
            port = await companion_spawner.spawn_companion(target_udid=target_udid)
            if port:
                self.logger.info(f"spawned a companion for {target_udid}")
                host = "localhost"
                companion_info = CompanionInfo(
                    host=host, port=port, udid=target_udid, is_local=True
                )
                self.direct_companion_manager.add_companion(companion_info)
                return companion_info
        return None

    @property
    def metadata(self) -> Dict[str, str]:
        if self.target_udid:
            return {"udid": self.target_udid}
        else:
            return {}

    async def kill(self) -> None:
        self.direct_companion_manager.clear()
        self.local_targets_manager.clear()
        PidSaver(logger=self.logger).kill_saved_pids()

    @log_and_handle_exceptions
    async def list_apps(self) -> List[InstalledAppInfo]:
        async with self.get_stub() as stub:
            response = await stub.list_apps(ListAppsRequest())
            return [
                InstalledAppInfo(
                    bundle_id=app.bundle_id,
                    name=app.name,
                    architectures=app.architectures,
                    install_type=app.install_type,
                    process_state=AppProcessState(app.process_state),
                    debuggable=app.debuggable,
                )
                for app in response.apps
            ]

    @log_and_handle_exceptions
    async def accessibility_info(
        self, point: Optional[Tuple[int, int]]
    ) -> AccessibilityInfo:
        async with self.get_stub() as stub:
            grpc_point = Point(x=point[0], y=point[1]) if point is not None else None
            response = await stub.accessibility_info(
                AccessibilityInfoRequest(point=grpc_point)
            )
            return AccessibilityInfo(json=response.json)

    @log_and_handle_exceptions
    async def add_media(self, file_paths: List[str]) -> None:
        async with self.get_stub() as stub, stub.add_media.open() as stream:
            if none_throws(self.companion_info).is_local:
                for file_path in file_paths:
                    await stream.send_message(
                        AddMediaRequest(payload=Payload(file_path=file_path))
                    )
                await stream.end()
                await stream.recv_message()
            else:
                generator = stream_map(
                    generate_tar(paths=file_paths, place_in_subfolders=True),
                    lambda chunk: AddMediaRequest(payload=Payload(data=chunk)),
                )
                await drain_to_stream(
                    stream=stream, generator=generator, logger=self.logger
                )

    @log_and_handle_exceptions
    async def approve(self, bundle_id: str, permissions: Set[str]) -> None:
        async with self.get_stub() as stub:
            await stub.approve(
                ApproveRequest(
                    bundle_id=bundle_id,
                    permissions=[APPROVE_MAP[permission] for permission in permissions],
                )
            )

    @log_and_handle_exceptions
    async def clear_keychain(self) -> None:
        async with self.get_stub() as stub:
            await stub.clear_keychain(ClearKeychainRequest())

    @log_and_handle_exceptions
    async def contacts_update(self, contacts_path: str) -> None:
        async with self.get_stub() as stub:
            data = await create_tar([contacts_path])
            await stub.contacts_update(
                ContactsUpdateRequest(payload=Payload(data=data))
            )

    @log_and_handle_exceptions
    async def screenshot(self) -> bytes:
        async with self.get_stub() as stub:
            response = await stub.screenshot(ScreenshotRequest())
            return response.image_data

    @log_and_handle_exceptions
    async def set_location(self, latitude: float, longitude: float) -> None:
        async with self.get_stub() as stub:
            await stub.set_location(
                SetLocationRequest(
                    location=Location(latitude=latitude, longitude=longitude)
                )
            )

    @log_and_handle_exceptions
    async def terminate(self, bundle_id: str) -> None:
        async with self.get_stub() as stub:
            await stub.terminate(TerminateRequest(bundle_id=bundle_id))

    @log_and_handle_exceptions
    async def describe(self) -> TargetDescription:
        async with self.get_stub() as stub:
            response = await stub.describe(TargetDescriptionRequest())
            return target_to_py(response.target_description)

    @log_and_handle_exceptions
    async def focus(self) -> None:
        async with self.get_stub() as stub:
            await stub.focus(FocusRequest())

    @log_and_handle_exceptions
    async def open_url(self, url: str) -> None:
        async with self.get_stub() as stub:
            await stub.open_url(OpenUrlRequest(url=url))

    @log_and_handle_exceptions
    async def uninstall(self, bundle_id: str) -> None:
        async with self.get_stub() as stub:
            await stub.uninstall(UninstallRequest(bundle_id=bundle_id))

    @log_and_handle_exceptions
    async def rm(self, bundle_id: str, paths: List[str]) -> None:
        async with self.get_stub() as stub:
            await stub.rm(RmRequest(bundle_id=bundle_id, paths=paths))

    @log_and_handle_exceptions
    async def mv(self, bundle_id: str, src_paths: List[str], dest_path: str) -> None:
        async with self.get_stub() as stub:
            await stub.mv(
                MvRequest(bundle_id=bundle_id, src_paths=src_paths, dst_path=dest_path)
            )

    @log_and_handle_exceptions
    async def ls(self, bundle_id: str, path: str) -> List[FileEntryInfo]:
        async with self.get_stub() as stub:
            response = await stub.ls(LsRequest(bundle_id=bundle_id, path=path))
            return [FileEntryInfo(path=file.path) for file in response.files]

    @log_and_handle_exceptions
    async def mkdir(self, bundle_id: str, path: str) -> None:
        async with self.get_stub() as stub:
            await stub.mkdir(MkdirRequest(bundle_id=bundle_id, path=path))

    @log_and_handle_exceptions
    async def crash_delete(self, query: CrashLogQuery) -> List[CrashLogInfo]:
        async with self.get_stub() as stub:
            response = await stub.crash_delete(_to_crash_log_query_proto(query))
            return _to_crash_log_info_list(response)

    @log_and_handle_exceptions
    async def crash_list(self, query: CrashLogQuery) -> List[CrashLogInfo]:
        async with self.get_stub() as stub:
            response = await stub.crash_list(_to_crash_log_query_proto(query))
            return _to_crash_log_info_list(response)

    @log_and_handle_exceptions
    async def crash_show(self, name: str) -> CrashLog:
        async with self.get_stub() as stub:
            response = await stub.crash_show(CrashShowRequest(name=name))
            return _to_crash_log(response)

    @log_and_handle_exceptions
    async def install(self, bundle: Bundle) -> InstalledArtifact:
        return await self._install_to_destination(
            bundle=bundle, destination=InstallRequest.APP
        )

    @log_and_handle_exceptions
    async def install_xctest(self, xctest: Bundle) -> InstalledArtifact:
        return await self._install_to_destination(
            bundle=xctest, destination=InstallRequest.XCTEST
        )

    @log_and_handle_exceptions
    async def install_dylib(self, dylib: Bundle) -> InstalledArtifact:
        return await self._install_to_destination(
            bundle=dylib, destination=InstallRequest.DYLIB
        )

    @log_and_handle_exceptions
    async def install_dsym(self, dsym: Bundle) -> InstalledArtifact:
        return await self._install_to_destination(
            bundle=dsym, destination=InstallRequest.DSYM
        )

    @log_and_handle_exceptions
    async def install_framework(self, framework_path: Bundle) -> InstalledArtifact:
        return await self._install_to_destination(
            bundle=framework_path, destination=InstallRequest.FRAMEWORK
        )

    async def _install_to_destination(
        self, bundle: Bundle, destination: Destination
    ) -> InstalledArtifact:
        async with self.get_stub() as stub, stub.install.open() as stream:
            generator = None
            if isinstance(bundle, str):
                url = urllib.parse.urlparse(bundle)
                if url.scheme:
                    # send url
                    payload = Payload(url=bundle)
                    generator = generate_requests([InstallRequest(payload=payload)])

                else:
                    file_path = str(Path(bundle).resolve(strict=True))
                    if none_throws(self.companion_info).is_local:
                        # send file_path
                        generator = generate_requests(
                            [InstallRequest(payload=Payload(file_path=file_path))]
                        )
                    else:
                        # chunk file from file_path
                        generator = generate_binary_chunks(
                            path=file_path, destination=destination, logger=self.logger
                        )

            else:
                # chunk file from memory
                generator = generate_io_chunks(io=bundle, logger=self.logger)
                # stream to companion
            await stream.send_message(InstallRequest(destination=destination))
            response = await drain_to_stream(
                stream=stream, generator=generator, logger=self.logger
            )
            return InstalledArtifact(name=response.name, uuid=response.uuid)

    @log_and_handle_exceptions
    async def push(self, src_paths: List[str], bundle_id: str, dest_path: str) -> None:
        async with self.get_stub() as stub, stub.push.open() as stream:
            await stream.send_message(
                PushRequest(
                    inner=PushRequest.Inner(bundle_id=bundle_id, dst_path=dest_path)
                )
            )
            if none_throws(self.companion_info).is_local:
                for src_path in src_paths:
                    await stream.send_message(
                        PushRequest(payload=Payload(file_path=src_path))
                    )
                await stream.end()
                await stream.recv_message()
            else:
                await drain_to_stream(
                    stream=stream,
                    generator=stream_map(
                        generate_tar(paths=src_paths),
                        lambda chunk: PushRequest(payload=Payload(data=chunk)),
                    ),
                    logger=self.logger,
                )

    @log_and_handle_exceptions
    async def pull(self, bundle_id: str, src_path: str, dest_path: str) -> None:
        async with self.get_stub() as stub, stub.pull.open() as stream:
            request = request = PullRequest(
                bundle_id=bundle_id,
                src_path=src_path,
                # not sending the destination to remote companion
                # so it streams the file back
                dst_path=dest_path
                if none_throws(self.companion_info).is_local
                else None,
            )
            await stream.send_message(request)
            await stream.end()
            if none_throws(self.companion_info).is_local:
                await stream.recv_message()
            else:
                await drain_untar(generate_bytes(stream), output_path=dest_path)
            self.logger.info(f"pulled file to {dest_path}")
        return PullResponse(payload=Payload(file_path=dest_path))

    @log_and_handle_exceptions
    async def list_test_bundle(self, test_bundle_id: str) -> List[str]:
        async with self.get_stub() as stub:
            response = await stub.xctest_list_tests(
                XctestListTestsRequest(bundle_name=test_bundle_id)
            )
            return [name for name in response.names]

    @log_and_handle_exceptions
    async def list_xctests(self) -> List[InstalledTestInfo]:
        async with self.get_stub() as stub:
            response = await stub.xctest_list_bundles(XctestListBundlesRequest())
            return [
                InstalledTestInfo(
                    bundle_id=bundle.bundle_id,
                    name=bundle.name,
                    architectures=bundle.architectures,
                )
                for bundle in response.bundles
            ]

    @log_and_handle_exceptions
    async def send_events(self, events: Iterable[HIDEvent]) -> None:
        await self.hid(iterator_to_async_iterator(events))

    @log_and_handle_exceptions
    async def tap(self, x: int, y: int, duration: Optional[float] = None) -> None:
        await self.send_events(tap_to_events(x, y, duration))

    @log_and_handle_exceptions
    async def button(
        self, button_type: HIDButtonType, duration: Optional[float] = None
    ) -> None:
        await self.send_events(button_press_to_events(button_type, duration))

    @log_and_handle_exceptions
    async def key(self, keycode: int, duration: Optional[float] = None) -> None:
        await self.send_events(key_press_to_events(keycode, duration))

    @log_and_handle_exceptions
    async def text(self, text: str) -> None:
        await self.send_events(text_to_events(text))

    @log_and_handle_exceptions
    async def swipe(
        self,
        p_start: Tuple[int, int],
        p_end: Tuple[int, int],
        delta: Optional[int] = None,
    ) -> None:
        await self.send_events(swipe_to_events(p_start, p_end, delta))

    @log_and_handle_exceptions
    async def key_sequence(self, key_sequence: List[int]) -> None:
        events: List[HIDEvent] = []
        for key in key_sequence:
            events.extend(key_press_to_events(key))
        await self.send_events(events)

    @log_and_handle_exceptions
    async def hid(self, event_iterator: AsyncIterable[HIDEvent]) -> None:
        async with self.get_stub() as stub, stub.hid.open() as stream:
            grpc_event_iterator = (
                event_to_grpc(event) async for event in event_iterator
            )
            await drain_to_stream(
                stream=stream, generator=grpc_event_iterator, logger=self.logger
            )
            await stream.recv_message()

    async def debug_server(self, request: DebugServerRequest) -> DebugServerResponse:
        async with self.get_stub() as stub, stub.debugserver.open() as stream:
            await stream.send_message(request)
            await stream.end()
            return await stream.recv_message()

    @log_and_handle_exceptions
    async def debugserver_start(self, bundle_id: str) -> List[str]:
        response = await self.debug_server(
            request=DebugServerRequest(
                start=DebugServerRequest.Start(bundle_id=bundle_id)
            )
        )
        return response.status.lldb_bootstrap_commands

    @log_and_handle_exceptions
    async def debugserver_stop(self) -> None:
        await self.debug_server(
            request=DebugServerRequest(stop=DebugServerRequest.Stop())
        )

    @log_and_handle_exceptions
    async def debugserver_status(self) -> Optional[List[str]]:
        response = await self.debug_server(
            request=DebugServerRequest(status=DebugServerRequest.Status())
        )
        commands = response.status.lldb_bootstrap_commands
        return commands if commands else None

    @log_and_handle_exceptions
    async def run_instruments(
        self,
        stop: asyncio.Event,
        template: str,
        app_bundle_id: str,
        trace_path: str,
        post_process_arguments: Optional[List[str]] = None,
        env: Optional[Dict[str, str]] = None,
        app_args: Optional[List[str]] = None,
        started: Optional[asyncio.Event] = None,
        timings: Optional[InstrumentsTimings] = None,
    ) -> str:
        trace_path = os.path.realpath(trace_path)
        self.logger.info(f"Starting instruments connection, writing to {trace_path}")
        async with self.get_stub() as stub, stub.instruments_run.open() as stream:
            self.logger.info("Sending instruments request")
            await stream.send_message(
                InstrumentsRunRequest(
                    start=InstrumentsRunRequest.Start(
                        file_path=None,
                        template_name=template,
                        app_bundle_id=app_bundle_id,
                        environment=env,
                        arguments=app_args,
                        timings=translate_instruments_timings(timings),
                    )
                )
            )
            self.logger.info("Starting instruments")
            await drain_until_running(stream=stream, logger=self.logger)
            if started:
                started.set()
            self.logger.info("Instruments has started, waiting for stop")
            async for response in stop_wrapper(stream=stream, stop=stop):
                output = response.log_output
                if len(output):
                    self.logger.info(output.decode())
            self.logger.info("Stopping instruments")
            await stream.send_message(
                InstrumentsRunRequest(
                    stop=InstrumentsRunRequest.Stop(
                        post_process_arguments=post_process_arguments
                    )
                )
            )
            await stream.end()
            self.logger.info(f"Writing instruments from tar to {trace_path}")
            await drain_untar(
                instruments_generate_bytes(stream=stream, logger=self.logger),
                output_path=trace_path,
            )
            self.logger.info(f"Instruments trace written to {trace_path}")
            return trace_path

    @log_and_handle_exceptions
    async def launch(
        self,
        bundle_id: str,
        args: Optional[List[str]] = None,
        env: Optional[Dict[str, str]] = None,
        foreground_if_running: bool = False,
        stop: Optional[asyncio.Event] = None,
    ) -> None:
        async with self.get_stub() as stub, stub.launch.open() as stream:
            request = LaunchRequest(
                start=LaunchRequest.Start(
                    bundle_id=bundle_id,
                    env=env,
                    app_args=args,
                    foreground_if_running=foreground_if_running,
                    wait_for=True if stop else False,
                )
            )
            await stream.send_message(request)
            if stop:
                await asyncio.gather(
                    drain_launch_stream(stream), end_launch_stream(stream, stop)
                )
            else:
                await stream.end()
                await drain_launch_stream(stream)

    @log_and_handle_exceptions
    async def record_video(self, stop: asyncio.Event, output_file: str) -> None:
        self.logger.info(f"Starting connection to backend")
        async with self.get_stub() as stub, stub.record.open() as stream:
            if none_throws(self.companion_info).is_local:
                self.logger.info(
                    f"Starting video recording to local file {output_file}"
                )
                await stream.send_message(
                    RecordRequest(start=RecordRequest.Start(file_path=output_file))
                )
            else:
                self.logger.info(f"Starting video recording with response data")
                await stream.send_message(
                    RecordRequest(start=RecordRequest.Start(file_path=None))
                )
            await stop.wait()
            self.logger.info("Stopping video recording")
            await stream.send_message(RecordRequest(stop=RecordRequest.Stop()))
            await stream.end()
            if none_throws(self.companion_info).is_local:
                self.logger.info("Video saved at output path")
                await stream.recv_message()
            else:
                self.logger.info(f"Decompressing gzip to {output_file}")
                await drain_gzip_decompress(
                    generate_video_bytes(stream), output_path=output_file
                )
                self.logger.info(f"Finished decompression to {output_file}")

    @log_and_handle_exceptions
    async def run_xctest(
        self,
        test_bundle_id: str,
        app_bundle_id: str,
        test_host_app_bundle_id: Optional[str] = None,
        is_ui_test: bool = False,
        is_logic_test: bool = False,
        tests_to_run: Optional[Set[str]] = None,
        tests_to_skip: Optional[Set[str]] = None,
        env: Optional[Dict[str, str]] = None,
        args: Optional[List[str]] = None,
        result_bundle_path: Optional[str] = None,
        idb_log_buffer: Optional[StringIO] = None,
        timeout: Optional[int] = None,
        poll_interval_sec: float = TESTS_POLL_INTERVAL,
    ) -> AsyncIterator[TestRunInfo]:
        async with self.get_stub() as stub, stub.xctest_run.open() as stream:
            request = make_request(
                test_bundle_id=test_bundle_id,
                app_bundle_id=app_bundle_id,
                test_host_app_bundle_id=test_host_app_bundle_id,
                is_ui_test=is_ui_test,
                is_logic_test=is_logic_test,
                tests_to_run=tests_to_run,
                tests_to_skip=tests_to_skip,
                env=env,
                args=args,
                result_bundle_path=result_bundle_path,
                timeout=timeout,
            )
            await stream.send_message(request)
            await stream.end()
            async for response in stream:
                # response.log_output is a container of strings.
                # google.protobuf.pyext._message.RepeatedScalarContainer.
                for line in [
                    line
                    for lines in response.log_output
                    for line in lines.splitlines(keepends=True)
                ]:
                    self.logger.info(line)
                    if idb_log_buffer:
                        idb_log_buffer.write(line)
                if result_bundle_path:
                    await write_result_bundle(
                        response=response,
                        output_path=result_bundle_path,
                        logger=self.logger,
                    )
                for result in make_results(response):
                    yield result

    async def _tail_specific_logs(
        self,
        source: LogRequest.Source,
        stop: asyncio.Event,
        arguments: Optional[List[str]],
    ) -> AsyncIterator[str]:
        async with self.get_stub() as stub, stub.log.open() as stream:
            await stream.send_message(
                LogRequest(arguments=arguments, source=source), end=True
            )
            async for message in cancel_wrapper(stream=stream, stop=stop):
                yield message.output.decode()

    @log_and_handle_exceptions
    async def tail_logs(
        self, stop: asyncio.Event, arguments: Optional[List[str]] = None
    ) -> AsyncIterator[str]:
        async for message in self._tail_specific_logs(
            source=LogRequest.TARGET, stop=stop, arguments=arguments
        ):
            yield message

    @log_and_handle_exceptions
    async def tail_companion_logs(self, stop: asyncio.Event) -> AsyncIterator[str]:
        async for message in self._tail_specific_logs(
            source=LogRequest.COMPANION, stop=stop, arguments=None
        ):
            yield message

    @log_and_handle_exceptions
    async def boot(self) -> None:
        if self.target_udid:
            cmd: List[str] = ["idb_companion", "--boot", none_throws(self.target_udid)]
            process = await asyncio.create_subprocess_exec(
                *cmd, stdout=asyncio.subprocess.PIPE, stderr=asyncio.subprocess.PIPE
            )
            await process.communicate()
        else:
            raise IdbException("boot needs --udid to work")

    async def _companion_to_target(self, companion: CompanionInfo) -> TargetDescription:
        channel = Channel(companion.host, companion.port, loop=asyncio.get_event_loop())
        stub = CompanionServiceStub(channel=channel)
        response = await stub.describe(TargetDescriptionRequest())
        channel.close()
        return target_to_py(response.target_description)

    @log_and_handle_exceptions
    async def list_targets(self) -> List[TargetDescription]:
        await self.spawn_notifier()
        companions = self.direct_companion_manager.get_companions()
        local_targets = self.local_targets_manager.get_local_targets()
        connected_targets = await asyncio.gather(
            *(
                self._companion_to_target(companion=companion)
                for companion in companions
            )
        )
        return local_targets + list(connected_targets)

    @log_and_handle_exceptions
    async def connect(
        self,
        destination: ConnectionDestination,
        metadata: Optional[Dict[str, str]] = None,
    ) -> CompanionInfo:
        self.logger.debug(f"Connecting directly to {destination} with meta {metadata}")
        if isinstance(destination, Address):
            channel = Channel(
                destination.host, destination.port, loop=asyncio.get_event_loop()
            )
            stub = CompanionServiceStub(channel=channel)
            response = await stub.connect(
                ConnectRequest(
                    destination=destination_to_grpc(destination), metadata=metadata
                )
            )
            companion = CompanionInfo(
                udid=response.companion.udid,
                host=destination.host,
                port=destination.port,
                is_local=response.companion.is_local,
            )
            self.logger.debug(f"Connected directly to {companion}")
            self.direct_companion_manager.add_companion(companion)
            channel.close()
            return companion
        else:
            companion = await self.spawn_companion(target_udid=destination)
            if companion:
                return companion
            else:
                raise IdbException(f"can't find target for udid {destination}")

    @log_and_handle_exceptions
    async def disconnect(self, destination: ConnectionDestination) -> None:
        self.direct_companion_manager.remove_companion(destination)



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

from argparse import ArgumentParser, Namespace

from idb.cli.commands.base import TargetCommand
from idb.common.types import IdbClient


class AccessibilityInfoAllCommand(TargetCommand):
    @property
    def description(self) -> str:
        return "Describes Accessibility Information for the entire screen"

    @property
    def name(self) -> str:
        return "describe-all"

    async def run_with_client(self, args: Namespace, client: IdbClient) -> None:
        info = await client.accessibility_info(point=None)
        print(info.json)


class AccessibilityInfoAtPointCommand(TargetCommand):
    @property
    def description(self) -> str:
        return "Describes Accessibility Information at a point on the screen"

    @property
    def name(self) -> str:
        return "describe-point"

    def add_parser_arguments(self, parser: ArgumentParser) -> None:
        parser.add_argument("x", help="The x-coordinate", type=int)
        parser.add_argument("y", help="The y-coordinate", type=int)
        super().add_parser_arguments(parser)

    async def run_with_client(self, args: Namespace, client: IdbClient) -> None:
        info = await client.accessibility_info(point=(args.x, args.y))
        print(info.json)



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

import asyncio
import os
from argparse import Namespace
from typing import Any
from unittest.mock import ANY, MagicMock, patch

from idb.cli.main import gen_main as cli_main
from idb.common.constants import XCTEST_TIMEOUT
from idb.common.types import CrashLogQuery, HIDButtonType, InstrumentsTimings
from idb.utils.testing import AsyncMock, TestCase


class TestParser(TestCase):
    def __init__(self, *args: Any, **kws: Any) -> None:
        super().__init__(*args, loop=asyncio.get_event_loop(), **kws)

    def setUp(self) -> None:
        self.client_mock = MagicMock(name="client_mock")
        self.client_patch = patch("idb.cli.commands.base.GrpcClient", self.client_mock)
        self.client_patch.start()
        self

    def tearDown(self) -> None:
        self.client_patch.stop()

    async def test_launch_with_udid(self) -> None:
        bundle_id = "com.foo.app"
        udid = "my udid"
        self.client_mock().launch = AsyncMock(return_value=bundle_id)
        await cli_main(cmd_input=["launch", "--udid", udid, bundle_id])
        self.client_mock().launch.assert_called_once_with(
            bundle_id=bundle_id, env={}, args=[], foreground_if_running=False, stop=None
        )

    async def test_boot(self) -> None:
        self.client_mock().boot = AsyncMock()
        udid = "my udid"
        await cli_main(cmd_input=["boot", "--udid", udid])
        self.client_mock().boot.assert_called_once()

    async def test_install(self) -> None:
        self.client_mock().install = AsyncMock(return_value="com.foo.bar")
        app_path = "testApp.app"
        await cli_main(cmd_input=["install", app_path])
        self.client_mock().install.assert_called_once_with(app_path)

    async def test_uninstall(self) -> None:
        self.client_mock().uninstall = AsyncMock()
        app_path = "com.dummy.app"
        await cli_main(cmd_input=["uninstall", app_path])
        self.client_mock().uninstall.assert_called_once_with(bundle_id=app_path)

    async def test_list_apps(self) -> None:
        self.client_mock().list_apps = AsyncMock(return_value=[])
        await cli_main(cmd_input=["list-apps"])
        self.client_mock().list_apps.assert_called_once()

    async def test_connect_with_host_and_port(self) -> None:
        self.client_mock().connect = AsyncMock()
        host = "someHost"
        port = 1234
        await cli_main(cmd_input=["connect", host, str(port)])
        self.client_mock().connect.assert_called_once_with(
            destination=(host, port), metadata=ANY
        )

    async def test_connect_with_udid(self) -> None:
        self.client_mock().connect = AsyncMock()
        udid = "0B3311FA-234C-4665-950F-37544F690B61"
        await cli_main(cmd_input=["connect", udid])
        self.client_mock().connect.assert_called_once_with(
            destination=udid, metadata=ANY
        )

    async def test_disconnect_with_host_and_port(self) -> None:
        self.client_mock().disconnect = AsyncMock()
        host = "someHost"
        port = 1234
        await cli_main(cmd_input=["disconnect", host, str(port)])
        self.client_mock().disconnect.assert_called_once_with(destination=(host, port))

    async def test_disconnect_with_udid(self) -> None:
        self.client_mock().disconnect = AsyncMock()
        udid = "0B3311FA-234C-4665-950F-37544F690B61"
        await cli_main(cmd_input=["disconnect", udid])
        self.client_mock().disconnect.assert_called_once_with(destination=udid)

    async def test_file_mkdir_flag(self) -> None:
        self.client_mock().mkdir = AsyncMock()
        src = "path"
        bundle_id = "com.bundle.id"
        cmd_input = ["file", "mkdir", src, "--bundle-id", bundle_id]
        await cli_main(cmd_input=cmd_input)
        self.client_mock().mkdir.assert_called_once_with(bundle_id=bundle_id, path=src)

    async def test_file_mkdir_colon(self) -> None:
        self.client_mock().mkdir = AsyncMock()
        cmd_input = ["file", "mkdir", "com.bundle.id:path"]
        await cli_main(cmd_input=cmd_input)
        self.client_mock().mkdir.assert_called_once_with(
            bundle_id="com.bundle.id", path="path"
        )

    async def test_file_rmpath_flag(self) -> None:
        self.client_mock().rm = AsyncMock()
        src = "path"
        bundle_id = "com.bundle.id"
        cmd_input = ["file", "remove", src, "--bundle-id", bundle_id]
        await cli_main(cmd_input=cmd_input)
        self.client_mock().rm.assert_called_once_with(bundle_id=bundle_id, paths=[src])

    async def test_file_rmpath_flag_multiple(self) -> None:
        self.client_mock().rm = AsyncMock()
        srcs = ["pathA", "pathB"]
        bundle_id = "com.bundle.id"
        cmd_input = ["file", "remove", srcs[0], srcs[1], "--bundle-id", bundle_id]
        await cli_main(cmd_input=cmd_input)
        self.client_mock().rm.assert_called_once_with(bundle_id=bundle_id, paths=srcs)

    async def test_file_rmpath_colon(self) -> None:
        self.client_mock().rm = AsyncMock()
        cmd_input = ["file", "remove", "com.bundle.id:path"]
        await cli_main(cmd_input=cmd_input)
        self.client_mock().rm.assert_called_once_with(
            bundle_id="com.bundle.id", paths=["path"]
        )

    async def test_file_listpath_flag(self) -> None:
        self.client_mock().ls = AsyncMock(return_value=[])
        src = "path"
        bundle_id = "com.bundle.id"
        cmd_input = ["file", "list", src, "--bundle-id", bundle_id]
        await cli_main(cmd_input=cmd_input)
        self.client_mock().ls.assert_called_once_with(bundle_id=bundle_id, path=src)

    async def test_file_listpath_colon(self) -> None:
        self.client_mock().ls = AsyncMock(return_value=[])
        cmd_input = ["file", "list", "com.bundle.id:path"]
        await cli_main(cmd_input=cmd_input)
        self.client_mock().ls.assert_called_once_with(
            bundle_id="com.bundle.id", path="path"
        )

    async def test_file_move_flag(self) -> None:
        self.client_mock().mv = AsyncMock(return_value=[])
        src = "a"
        dst = "b"
        bundle_id = "com.bundle.id"
        cmd_input = ["file", "move", src, dst, "--bundle-id", bundle_id]
        await cli_main(cmd_input=cmd_input)
        self.client_mock().mv.assert_called_once_with(
            bundle_id=bundle_id, src_paths=[src], dest_path=dst
        )

    async def test_file_move_flag_multiple(self) -> None:
        self.client_mock().mv = AsyncMock(return_value=[])
        srcs = ["src1", "src2"]
        dst = "b"
        bundle_id = "com.bundle.id"
        cmd_input = ["file", "move", srcs[0], srcs[1], dst, "--bundle-id", bundle_id]
        await cli_main(cmd_input=cmd_input)
        self.client_mock().mv.assert_called_once_with(
            bundle_id=bundle_id, src_paths=srcs, dest_path=dst
        )

    async def test_file_move_colon(self) -> None:
        self.client_mock().mv = AsyncMock(return_value=[])
        src = "a"
        cmd_input = ["file", "move", src, "com.bundle.id:b"]
        await cli_main(cmd_input=cmd_input)
        self.client_mock().mv.assert_called_once_with(
            bundle_id="com.bundle.id", src_paths=[src], dest_path="b"
        )

    async def test_file_push_single_flag(self) -> None:
        self.client_mock().push = AsyncMock()
        bundle_id = "com.myapp"
        src = "Library/myFile.txt"
        dst = "someOutputDir"
        cmd_input = ["file", "push", src, dst, "--bundle-id", bundle_id]
        await cli_main(cmd_input=cmd_input)
        self.client_mock().push.assert_called_once_with(
            bundle_id=bundle_id, src_paths=[os.path.abspath(src)], dest_path=dst
        )

    async def test_file_push_multi_flag(self) -> None:
        self.client_mock().push = AsyncMock(return_value=[])
        bundle_id = "com.myapp"
        src1 = "Library/myFile.txt"
        src2 = "Library/myFile2.txt"
        dst = "someOutputDir"
        cmd_input = ["file", "push", src1, src2, dst, "--bundle-id", bundle_id]
        await cli_main(cmd_input=cmd_input)
        self.client_mock().push.assert_called_once_with(
            bundle_id=bundle_id,
            src_paths=[os.path.abspath(path) for path in [src1, src2]],
            dest_path=dst,
        )

    async def test_file_push_single_colon(self) -> None:
        self.client_mock().push = AsyncMock(return_value=[])
        src = "Library/myFile.txt"
        cmd_input = ["file", "push", src, "com.bundle.id:someOutputDir"]
        await cli_main(cmd_input=cmd_input)
        self.client_mock().push.assert_called_once_with(
            bundle_id="com.bundle.id",
            src_paths=[os.path.abspath(src)],
            dest_path="someOutputDir",
        )

    async def test_file_push_multi_colon(self) -> None:
        self.client_mock().push = AsyncMock(return_value=[])
        src1 = "Library/myFile.txt"
        src2 = "Library/myFile2.txt"
        cmd_input = ["file", "push", src1, src2, "com.bundle.id:someOutputDir"]
        await cli_main(cmd_input=cmd_input)
        self.client_mock().push.assert_called_once_with(
            bundle_id="com.bundle.id",
            src_paths=[os.path.abspath(path) for path in [src1, src2]],
            dest_path="someOutputDir",
        )

    async def test_file_pull(self) -> None:
        self.client_mock().pull = AsyncMock(return_value=[])
        bundle_id = "com.myapp"
        src = "Library/myFile.txt"
        dst = "someOutputDir"
        cmd_input = ["file", "pull", src, dst, "--bundle-id", bundle_id]
        await cli_main(cmd_input=cmd_input)
        self.client_mock().pull.assert_called_once_with(
            bundle_id=bundle_id, src_path=src, dest_path=os.path.abspath(dst)
        )

    async def test_push_deprecated(self) -> None:
        self.client_mock().push = AsyncMock(return_value=[])
        src_path = "./fileToSend.txt"
        app_bundle_id = "com.myapp"
        dest_path = "Library"
        await cli_main(cmd_input=["push", src_path, app_bundle_id, dest_path])
        self.client_mock().push.assert_called_once_with(
            bundle_id=app_bundle_id,
            src_paths=[os.path.abspath(src_path)],
            dest_path=dest_path,
        )

    async def test_pull_deprecated(self) -> None:
        self.client_mock().pull = AsyncMock(return_value=[])
        bundle_id = "com.myapp"
        src = "Library/myFile.txt"
        dest = "someOutputDir"
        await cli_main(cmd_input=["pull", bundle_id, src, dest])
        self.client_mock().pull.assert_called_once_with(
            bundle_id=bundle_id, src_path=src, dest_path=os.path.abspath(dest)
        )

    async def test_list_targets(self) -> None:
        self.client_mock().list_targets = AsyncMock(return_value=[])
        await cli_main(cmd_input=["list-targets"])
        self.client_mock().list_targets.assert_called_once()

    async def test_kill(self) -> None:
        self.client_mock().kill = AsyncMock(return_value=[])
        await cli_main(cmd_input=["kill"])
        self.client_mock().kill.assert_called_once_with()

    async def test_xctest_install(self) -> None:
        self.client_mock().install_xctest = AsyncMock(return_value=[])
        test_bundle_path = "testBundle.xctest"
        await cli_main(cmd_input=["xctest", "install", test_bundle_path])
        self.client_mock().install_xctest.assert_called_once_with(test_bundle_path)

    def xctest_run_namespace(self, command: str, test_bundle_id: str) -> Namespace:
        namespace = Namespace()
        # pyre-fixme[16]: `Namespace` has no attribute `log_level`.
        namespace.log_level = "WARNING"
        # pyre-fixme[16]: `Namespace` has no attribute `root_command`.
        namespace.root_command = "xctest"
        # pyre-fixme[16]: `Namespace` has no attribute `xctest`.
        namespace.xctest = "run"
        # pyre-fixme[16]: `Namespace` has no attribute `udid`.
        namespace.udid = None
        # pyre-fixme[16]: `Namespace` has no attribute `json`.
        namespace.json = False
        if command in ["app", "ui"]:
            # pyre-fixme[16]: `Namespace` has no attribute `tests_to_run`.
            namespace.tests_to_run = None
            # pyre-fixme[16]: `Namespace` has no attribute `tests_to_skip`.
            namespace.tests_to_skip = None
            # pyre-fixme[16]: `Namespace` has no attribute `test_arguments`.
            namespace.test_arguments = []
        elif command == "logic":
            # pyre-fixme[16]: `Namespace` has no attribute `test_to_run`.
            namespace.test_to_run = None
        # pyre-fixme[16]: `Namespace` has no attribute `run`.
        namespace.run = command
        # pyre-fixme[16]: `Namespace` has no attribute `test_bundle_id`.
        namespace.test_bundle_id = test_bundle_id
        # pyre-fixme[16]: `Namespace` has no attribute `daemon_port`.
        namespace.daemon_port = 9888
        # pyre-fixme[16]: `Namespace` has no attribute `daemon_grpc_port`.
        namespace.daemon_grpc_port = 9889
        # pyre-fixme[16]: `Namespace` has no attribute `daemon_host`.
        namespace.daemon_host = "localhost"
        # pyre-fixme[16]: `Namespace` has no attribute `result_bundle_path`.
        namespace.result_bundle_path = None
        return namespace

    async def test_xctest_run_app(self) -> None:
        mock = AsyncMock()
        mock.return_value = []
        with patch(
            "idb.cli.commands.xctest.XcestRunAppCommand.run", new=mock, create=True
        ):
            test_bundle_id = "com.me.tests"
            app_under_test_id = "com.me.app"
            await cli_main(
                cmd_input=["xctest", "run", "app", test_bundle_id, app_under_test_id]
            )
            namespace = self.xctest_run_namespace("app", test_bundle_id)
            # pyre-fixme[16]: `Namespace` has no attribute `app_bundle_id`.
            namespace.app_bundle_id = app_under_test_id
            # pyre-fixme[16]: `Namespace` has no attribute `force`.
            namespace.force = False
            # pyre-fixme[16]: `Namespace` has no attribute `timeout`.
            namespace.timeout = XCTEST_TIMEOUT
            mock.assert_called_once_with(namespace)

    async def test_xctest_run_ui(self) -> None:
        mock = AsyncMock()
        mock.return_value = []
        with patch(
            "idb.cli.commands.xctest.XctestRunUICommand.run", new=mock, create=True
        ):
            test_bundle_id = "com.me.tests"
            app_under_test_id = "com.me.app"
            test_host_app_bundle_id = "com.host.app"
            await cli_main(
                cmd_input=[
                    "xctest",
                    "run",
                    "ui",
                    test_bundle_id,
                    app_under_test_id,
                    test_host_app_bundle_id,
                ]
            )
            namespace = self.xctest_run_namespace("ui", test_bundle_id)
            # pyre-fixme[16]: `Namespace` has no attribute `app_bundle_id`.
            namespace.app_bundle_id = app_under_test_id
            # pyre-fixme[16]: `Namespace` has no attribute `test_host_app_bundle_id`.
            namespace.test_host_app_bundle_id = test_host_app_bundle_id
            # pyre-fixme[16]: `Namespace` has no attribute `force`.
            namespace.force = False
            # pyre-fixme[16]: `Namespace` has no attribute `timeout`.
            namespace.timeout = XCTEST_TIMEOUT
            mock.assert_called_once_with(namespace)

    async def test_xctest_run_logic(self) -> None:
        mock = AsyncMock()
        mock.return_value = []
        with patch(
            "idb.cli.commands.xctest.CommonRunXcTestCommand.run", new=mock, create=True
        ):
            test_bundle_id = "com.me.tests"
            await cli_main(cmd_input=["xctest", "run", "logic", test_bundle_id])
            namespace = self.xctest_run_namespace("logic", test_bundle_id)
            # pyre-fixme[16]: `Namespace` has no attribute `timeout`.
            namespace.timeout = XCTEST_TIMEOUT
            # pyre-fixme[16]: `Namespace` has no attribute `force`.
            namespace.force = False
            mock.assert_called_once_with(namespace)

    async def test_xctest_list(self) -> None:
        self.client_mock().list_xctests = AsyncMock(return_value=[])
        await cli_main(cmd_input=["xctest", "list"])
        self.client_mock().list_xctests.assert_called_once()

    async def test_xctest_list_bundles(self) -> None:
        self.client_mock().list_test_bundle = AsyncMock(return_value=[])
        bundle_id = "myBundleID"
        await cli_main(cmd_input=["xctest", "list-bundle", bundle_id])
        self.client_mock().list_test_bundle.assert_called_once_with(
            test_bundle_id=bundle_id
        )

    async def test_daemon(self) -> None:
        mock = AsyncMock()
        with patch(
            "idb.cli.commands.daemon.DaemonCommand._run_impl", new=mock, create=True
        ):
            port = 9888
            grpc_port = 1235
            await cli_main(cmd_input=["daemon", "--daemon-grpc-port", str(grpc_port)])
            namespace = Namespace()
            # pyre-fixme[16]: `Namespace` has no attribute `daemon_port`.
            namespace.daemon_port = port
            # pyre-fixme[16]: `Namespace` has no attribute `daemon_grpc_port`.
            namespace.daemon_grpc_port = grpc_port
            # pyre-fixme[16]: `Namespace` has no attribute `log_level`.
            namespace.log_level = "WARNING"
            # pyre-fixme[16]: `Namespace` has no attribute `root_command`.
            namespace.root_command = "daemon"
            # pyre-fixme[16]: `Namespace` has no attribute `json`.
            namespace.json = False
            # pyre-fixme[16]: `Namespace` has no attribute `reply_fd`.
            namespace.reply_fd = None
            # pyre-fixme[16]: `Namespace` has no attribute `prefer_ipv6`.
            namespace.prefer_ipv6 = False
            # pyre-fixme[16]: `Namespace` has no attribute `notifier_path`.
            namespace.notifier_path = None
            mock.assert_called_once_with(namespace)

    async def test_terminate(self) -> None:
        self.client_mock().terminate = AsyncMock(return_value=[])
        bundle_id = "com.foo.app"
        udid = "my udid"
        await cli_main(cmd_input=["terminate", bundle_id, "--udid", udid])
        self.client_mock().terminate.assert_called_once_with(bundle_id)

    async def test_log(self) -> None:
        mock = AsyncMock()
        with patch("idb.cli.commands.log.LogCommand._run_impl", new=mock, create=True):
            await cli_main(cmd_input=["log", "--udid", "1234"])
            namespace = Namespace()
            # pyre-fixme[16]: `Namespace` has no attribute `log_level`.
            namespace.log_level = "WARNING"
            # pyre-fixme[16]: `Namespace` has no attribute `root_command`.
            namespace.root_command = "log"
            # pyre-fixme[16]: `Namespace` has no attribute `udid`.
            namespace.udid = "1234"
            # pyre-fixme[16]: `Namespace` has no attribute `json`.
            namespace.json = False
            # pyre-fixme[16]: `Namespace` has no attribute `log_arguments`.
            namespace.log_arguments = []
            # pyre-fixme[16]: `Namespace` has no attribute `daemon_port`.
            namespace.daemon_port = 9888
            # pyre-fixme[16]: `Namespace` has no attribute `daemon_grpc_port`.
            namespace.daemon_grpc_port = 9889
            # pyre-fixme[16]: `Namespace` has no attribute `daemon_host`.
            namespace.daemon_host = "localhost"
            # pyre-fixme[16]: `Namespace` has no attribute `force`.
            namespace.force = False
            mock.assert_called_once_with(namespace)

    async def test_log_arguments(self) -> None:
        mock = AsyncMock()
        with patch("idb.cli.commands.log.LogCommand._run_impl", new=mock, create=True):
            await cli_main(cmd_input=["log", "--", "--style", "json"])
            namespace = Namespace()
            # pyre-fixme[16]: `Namespace` has no attribute `log_level`.
            namespace.log_level = "WARNING"
            # pyre-fixme[16]: `Namespace` has no attribute `root_command`.
            namespace.root_command = "log"
            # pyre-fixme[16]: `Namespace` has no attribute `udid`.
            namespace.udid = None
            # pyre-fixme[16]: `Namespace` has no attribute `json`.
            namespace.json = False
            # pyre-fixme[16]: `Namespace` has no attribute `log_arguments`.
            namespace.log_arguments = ["--", "--style", "json"]
            # pyre-fixme[16]: `Namespace` has no attribute `daemon_port`.
            namespace.daemon_port = 9888
            # pyre-fixme[16]: `Namespace` has no attribute `daemon_grpc_port`.
            namespace.daemon_grpc_port = 9889
            # pyre-fixme[16]: `Namespace` has no attribute `daemon_host`.
            namespace.daemon_host = "localhost"
            # pyre-fixme[16]: `Namespace` has no attribute `force`.
            namespace.force = False
            mock.assert_called_once_with(namespace)

    async def test_clear_keychain(self) -> None:
        self.client_mock().clear_keychain = AsyncMock(return_value=[])
        await cli_main(cmd_input=["clear-keychain"])
        self.client_mock().clear_keychain.assert_called_once()

    async def test_open_url(self) -> None:
        self.client_mock().open_url = AsyncMock(return_value=[])
        url = "http://facebook.com"
        await cli_main(cmd_input=["open", url])
        self.client_mock().open_url.assert_called_once_with(url)

    async def test_set_location(self) -> None:
        self.client_mock().set_location = AsyncMock(return_value=[])
        latitude = 1.0
        longitude = 2.0
        await cli_main(cmd_input=["set-location", str(latitude), str(longitude)])
        self.client_mock().set_location.assert_called_once_with(latitude, longitude)

    async def test_approve(self) -> None:
        self.client_mock().approve = AsyncMock(return_value=[])
        bundle_id = "com.fb.myApp"
        await cli_main(cmd_input=["approve", bundle_id, "photos"])
        self.client_mock().approve.assert_called_once_with(
            bundle_id=bundle_id, permissions={"photos"}
        )

    async def test_video(self) -> None:
        mock = AsyncMock()
        with patch(
            "idb.cli.commands.record.RecordVideoCommand._run_impl",
            new=mock,
            create=True,
        ):
            output_file = "video.mp4"
            await cli_main(cmd_input=["record-video", output_file])
            namespace = Namespace()
            # pyre-fixme[16]: `Namespace` has no attribute `log_level`.
            namespace.log_level = "WARNING"
            # pyre-fixme[16]: `Namespace` has no attribute `root_command`.
            namespace.root_command = "record-video"
            # pyre-fixme[16]: `Namespace` has no attribute `udid`.
            namespace.udid = None
            # pyre-fixme[16]: `Namespace` has no attribute `json`.
            namespace.json = False
            # pyre-fixme[16]: `Namespace` has no attribute `output_file`.
            namespace.output_file = output_file
            # pyre-fixme[16]: `Namespace` has no attribute `daemon_port`.
            namespace.daemon_port = 9888
            # pyre-fixme[16]: `Namespace` has no attribute `daemon_grpc_port`.
            namespace.daemon_grpc_port = 9889
            # pyre-fixme[16]: `Namespace` has no attribute `daemon_host`.
            namespace.daemon_host = "localhost"
            # pyre-fixme[16]: `Namespace` has no attribute `force`.
            namespace.force = False
            mock.assert_called_once_with(namespace)

    async def test_key_sequence(self) -> None:
        self.client_mock().key_sequence = AsyncMock(return_value=[])
        await cli_main(cmd_input=["ui", "key-sequence", "1", "2", "3"])
        self.client_mock().key_sequence.assert_called_once_with(key_sequence=[1, 2, 3])

    async def test_tap(self) -> None:
        self.client_mock().tap = AsyncMock(return_value=[])
        await cli_main(cmd_input=["ui", "tap", "10", "20"])
        self.client_mock().tap.assert_called_once_with(x=10, y=20, duration=None)

    async def test_button(self) -> None:
        self.client_mock().button = AsyncMock(return_value=[])
        await cli_main(cmd_input=["ui", "button", "SIRI"])
        self.client_mock().button.assert_called_once_with(
            button_type=HIDButtonType.SIRI, duration=None
        )

    async def test_key(self) -> None:
        self.client_mock().key = AsyncMock(return_value=[])
        await cli_main(cmd_input=["ui", "key", "12"])
        self.client_mock().key.assert_called_once_with(keycode=12, duration=None)

    async def test_text_input(self) -> None:
        self.client_mock().text = AsyncMock(return_value=[])
        text = "Some Text"
        await cli_main(cmd_input=["ui", "text", text])
        self.client_mock().text.assert_called_once_with(text=text)

    async def test_swipe_with_delta(self) -> None:
        self.client_mock().swipe = AsyncMock(return_value=[])
        await cli_main(cmd_input=["ui", "swipe", "1", "2", "3", "4", "--delta", "5"])
        self.client_mock().swipe.assert_called_once_with(
            p_start=(1, 2), p_end=(3, 4), delta=5
        )

    async def test_swipe_without_delta(self) -> None:
        self.client_mock().swipe = AsyncMock(return_value=[])
        await cli_main(cmd_input=["ui", "swipe", "1", "2", "3", "4"])
        self.client_mock().swipe.assert_called_once_with(
            p_start=(1, 2), p_end=(3, 4), delta=None
        )

    async def test_contacts_update(self) -> None:
        self.client_mock().contacts_update = AsyncMock(return_value=[])
        await cli_main(cmd_input=["contacts", "update", "/dev/null"])
        self.client_mock().contacts_update.assert_called_once_with(
            contacts_path="/dev/null"
        )

    async def test_accessibility_info_all(self) -> None:
        self.client_mock().accessibility_info = AsyncMock()
        await cli_main(cmd_input=["ui", "describe-all"])
        self.client_mock().accessibility_info.assert_called_once_with(point=None)

    async def test_accessibility_info_at_point(self) -> None:
        self.client_mock().accessibility_info = AsyncMock()
        await cli_main(cmd_input=["ui", "describe-point", "10", "20"])
        self.client_mock().accessibility_info.assert_called_once_with(point=(10, 20))

    async def test_crash_list_all(self) -> None:
        self.client_mock().crash_list = AsyncMock(return_value=[])
        await cli_main(cmd_input=["crash", "list"])
        self.client_mock().crash_list.assert_called_once_with(query=CrashLogQuery())

    async def test_crash_list_with_predicate(self) -> None:
        self.client_mock().crash_list = AsyncMock(return_value=[])
        await cli_main(cmd_input=["crash", "list", "--since", "20"])
        self.client_mock().crash_list.assert_called_once_with(
            query=CrashLogQuery(since=20)
        )

    async def test_crash_show(self) -> None:
        self.client_mock().crash_show = AsyncMock()
        await cli_main(cmd_input=["crash", "show", "foo"])
        self.client_mock().crash_show.assert_called_once_with(name="foo")

    async def test_crash_delete_all(self) -> None:
        self.client_mock().crash_delete = AsyncMock(return_value=[])
        await cli_main(cmd_input=["crash", "delete", "--all"])
        self.client_mock().crash_delete.assert_called_once_with(query=CrashLogQuery())

    async def test_crash_delete_with_predicate(self) -> None:
        self.client_mock().crash_delete = AsyncMock(return_value=[])
        await cli_main(cmd_input=["crash", "delete", "--since", "20"])
        self.client_mock().crash_delete.assert_called_once_with(
            query=CrashLogQuery(since=20)
        )

    async def test_crash_delete_with_name(self) -> None:
        self.client_mock().crash_delete = AsyncMock(return_value=[])
        await cli_main(cmd_input=["crash", "delete", "some.foo.bar.crash"])
        self.client_mock().crash_delete.assert_called_once_with(
            query=CrashLogQuery(name="some.foo.bar.crash")
        )

    async def test_instruments(self) -> None:
        self.client_mock().run_instruments = AsyncMock()
        template_name = "System Trace"
        trace_path = template_name + ".trace"
        await cli_main(cmd_input=["instruments", template_name])
        self.client_mock().run_instruments.assert_called_once_with(
            stop=ANY,
            template=template_name,
            app_bundle_id=None,
            post_process_arguments=None,
            trace_path=trace_path,
            env={},
            app_args=None,
            timings=None,
        )

    async def test_instruments_with_post_args(self) -> None:
        self.client_mock().run_instruments = AsyncMock()
        template_name = "Time Profiler"
        trace_path = template_name + ".trace"
        post_process_arguments = ["instrumental", "convert"]
        await cli_main(
            cmd_input=[
                "instruments",
                template_name,
                "--post-args",
                *post_process_arguments,
            ]
        )
        self.client_mock().run_instruments.assert_called_once_with(
            stop=ANY,
            template=template_name,
            app_bundle_id=None,
            post_process_arguments=post_process_arguments,
            trace_path=trace_path,
            env={},
            app_args=None,
            timings=None,
        )

    async def test_instruments_with_app_args(self) -> None:
        self.client_mock().run_instruments = AsyncMock()
        template_name = "System Trace"
        trace_path = template_name + ".trace"
        await cli_main(
            cmd_input=["instruments", template_name, "--app-args", "perfLab"]
        )
        self.client_mock().run_instruments.assert_called_once_with(
            stop=ANY,
            template=template_name,
            app_bundle_id=None,
            post_process_arguments=None,
            trace_path=trace_path,
            env={},
            app_args=["perfLab"],
            timings=None,
        )

    async def test_instruments_with_all_timings(self) -> None:
        self.client_mock().run_instruments = AsyncMock()
        template_name = "System Trace"
        trace_path = template_name + ".trace"
        (
            launch_error_timeout,
            launch_retry_timeout,
            terminate_timeout,
            operation_duration,
        ) = ("10.0", "200.0", "200.0", "30")
        await cli_main(
            cmd_input=[
                "instruments",
                template_name,
                "--app-args",
                "perfLab",
                "--operation-duration",
                operation_duration,
                "--launch-retry-timeout",
                launch_retry_timeout,
                "--terminate-timeout",
                terminate_timeout,
                "--launch-error-timeout",
                launch_error_timeout,
            ]
        )
        self.client_mock().run_instruments.assert_called_once_with(
            stop=ANY,
            template=template_name,
            app_bundle_id=None,
            post_process_arguments=None,
            trace_path=trace_path,
            env={},
            app_args=["perfLab"],
            timings=InstrumentsTimings(
                launch_error_timeout=float(launch_error_timeout),
                launch_retry_timeout=float(launch_retry_timeout),
                terminate_timeout=float(terminate_timeout),
                operation_duration=float(operation_duration),
            ),
        )

    async def test_instruments_with_partial_timings(self) -> None:
        self.client_mock().run_instruments = AsyncMock()
        template_name = "System Trace"
        trace_path = template_name + ".trace"
        launch_retry_timeout, terminate_timeout = ("200.0", "200.0")
        await cli_main(
            cmd_input=[
                "instruments",
                template_name,
                "--app-args",
                "perfLab",
                "--launch-retry-timeout",
                launch_retry_timeout,
                "--terminate-timeout",
                terminate_timeout,
            ]
        )
        self.client_mock().run_instruments.assert_called_once_with(
            stop=ANY,
            template=template_name,
            app_bundle_id=None,
            post_process_arguments=None,
            trace_path=trace_path,
            env={},
            app_args=["perfLab"],
            timings=InstrumentsTimings(
                launch_error_timeout=None,
                launch_retry_timeout=float(launch_retry_timeout),
                terminate_timeout=float(terminate_timeout),
                operation_duration=None,
            ),
        )

    async def test_add_media(self) -> None:
        self.client_mock().add_media = AsyncMock(return_value=["aaa", "bbb"])
        file_paths = ["cat.jpeg", "dog.mov"]
        await cli_main(cmd_input=["add-media"] + file_paths)
        self.client_mock().add_media.assert_called_once_with(file_paths=file_paths)

    async def test_focus(self) -> None:
        self.client_mock().focus = AsyncMock(return_value=["aaa", "bbb"])
        await cli_main(cmd_input=["focus"])
        self.client_mock().focus.assert_called_once()

    async def test_debugserver_start(self) -> None:
        self.client_mock().debugserver_start = AsyncMock(return_value=["aaa", "bbb"])
        await cli_main(cmd_input=["debugserver", "start", "com.foo.bar"])
        self.client_mock().debugserver_start.assert_called_once_with(
            bundle_id="com.foo.bar"
        )

    async def test_debugserver_stop(self) -> None:
        self.client_mock().debugserver_stop = AsyncMock(return_value=["aaa", "bbb"])
        await cli_main(cmd_input=["debugserver", "stop"])
        self.client_mock().debugserver_stop.assert_called_once()

    async def test_debugserver_status(self) -> None:
        self.client_mock().debugserver_status = AsyncMock(return_value=["aaa", "bbb"])
        await cli_main(cmd_input=["debugserver", "status"])
        self.client_mock().debugserver_status.assert_called_once()



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

from argparse import Namespace

from idb.cli.commands.base import TargetCommand
from idb.common.types import IdbClient


class BootCommand(TargetCommand):
    @property
    def description(self) -> str:
        return "Boots a simulator (only works on mac)"

    @property
    def name(self) -> str:
        return "boot"

    async def run_with_client(self, args: Namespace, client: IdbClient) -> None:
        await client.boot()



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

import json
import logging
import os
from contextlib import contextmanager
from datetime import datetime, timedelta
from time import sleep
from typing import AsyncGenerator, List, Optional

from idb.common.constants import IDB_STATE_FILE_PATH
from idb.common.format import json_data_companions, json_to_companion_info
from idb.common.types import CompanionInfo, ConnectionDestination, IdbException


# this is the new companion manager for direct_client mode


@contextmanager
def exclusive_open(
    filename: str, *args, **kwargs  # pyre-ignore  # pyre-ignore
) -> AsyncGenerator[None, None]:
    timeout = 3
    retry_time = 0.05
    lockfile = filename + ".lock"
    deadline = datetime.now() + timedelta(seconds=timeout)
    while True:
        try:
            fd = os.open(lockfile, os.O_CREAT | os.O_EXCL)
            break
        except FileExistsError:
            if datetime.now() >= deadline:
                raise
            sleep(retry_time)
    try:
        with open(filename, *args, **kwargs) as f:
            yield f
    finally:
        try:
            os.close(fd)  # pyre-ignore
        finally:
            os.unlink(lockfile)


class DirectCompanionManager:
    def __init__(
        self, logger: logging.Logger, state_file_path: str = IDB_STATE_FILE_PATH
    ) -> None:
        self.companions: List[CompanionInfo] = []
        self.state_file_path = state_file_path
        self.logger = logger
        self.logger.info(f"idb state file stored at {self.state_file_path}")

    def add_companion(self, companion: CompanionInfo) -> None:
        self.get_companions()
        if companion in self.companions:
            self.logger.info(f"companion {companion} already added")
        else:
            self.companions.append(companion)
            self.logger.info(f"added direct companion {companion}")
        self._save()

    def get_companions(self) -> List[CompanionInfo]:
        self.companions = self._load()
        return self.companions

    def _save(self) -> None:
        with exclusive_open(self.state_file_path, "w") as f:
            json.dump(json_data_companions(self.companions), f)

    def _load(self) -> List[CompanionInfo]:
        if os.path.exists(self.state_file_path):
            with exclusive_open(self.state_file_path, "r") as f:
                return json_to_companion_info(json.load(f))
        return []

    def clear(self) -> None:
        self.companions = []
        self._save()

    def get_companion_info(self, target_udid: Optional[str]) -> CompanionInfo:
        self.get_companions()
        if target_udid:
            companions = [
                companion
                for companion in self.companions
                if companion.udid == target_udid
            ]
            if len(companions) > 0:
                return companions[0]
            else:
                raise IdbException(
                    f"Couldn't find companion for target with udid {target_udid}"
                )
        elif len(self.companions) >= 1:
            companion = self.companions[0]
            self.logger.info(f"using default companion with udid {companion.udid}")
            return companion
        else:
            raise IdbException("No UDID provided and couldn't find a default companion")

    def remove_companion(self, destination: ConnectionDestination) -> None:
        self.get_companions()
        companions = []
        if isinstance(destination, str):
            companions = [
                companion
                for companion in self.companions
                if companion.udid == destination
            ]
        else:
            companions = [
                companion
                for companion in self.companions
                if companion.host == destination.host
                and companion.port == destination.port
            ]
        for companion in companions:
            self.companions.remove(companion)
        self._save()



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

import logging
import warnings
from typing import Dict, Optional

from idb.common.direct_companion_manager import DirectCompanionManager
from idb.grpc.idb_grpc import CompanionServiceBase


# Don't let the abstractmetod machineary mess raise at runtime
CompanionServiceBase.__abstractmethods__ = frozenset([])
# this is to silence the channel not closed warning
# https://github.com/vmagamedov/grpclib/issues/58
warnings.filterwarnings(action="ignore", category=ResourceWarning)


class GRPCHandler(CompanionServiceBase):
    def __init__(self, logger: Optional[logging.Logger] = None) -> None:
        self.logger: logging.Logger = (
            logger if logger else logging.getLogger("idb_daemon")
        )
        self.direct_companion_manager = DirectCompanionManager(logger=self.logger)

    def get_udid(self, metadata: Dict[str, str]) -> Optional[str]:
        return metadata.get("udid")



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

from idb.ipc.mapping.hid import (
    GrpcHIDButton,
    GrpcHIDDelay,
    GrpcHIDEvent,
    GrpcHIDKey,
    GrpcHIDPress,
    GrpcHIDPressAction,
    GrpcHIDSwipe,
    GrpcHIDTouch,
    GrpcPoint,
    HIDButton,
    HIDButtonType,
    HIDDelay,
    HIDDirection,
    HIDKey,
    HIDPress,
    HIDSwipe,
    HIDTouch,
    Point,
    event_to_grpc,
)
from idb.utils.testing import TestCase


class HidTests(TestCase):
    def test_press(self) -> None:
        actions = [
            HIDTouch(point=Point(x=1, y=2)),
            HIDButton(button=HIDButtonType.HOME),
            HIDKey(keycode=3),
        ]
        expected = [
            GrpcHIDPressAction(touch=GrpcHIDTouch(point=GrpcPoint(x=1, y=2))),
            GrpcHIDPressAction(button=GrpcHIDButton(button=GrpcHIDEvent.HOME)),
            GrpcHIDPressAction(key=GrpcHIDKey(keycode=3)),
        ]
        for action, expected in zip(actions, expected):
            self.assertEqual(
                event_to_grpc(HIDPress(action=action, direction=HIDDirection.UP)),
                GrpcHIDEvent(
                    press=GrpcHIDPress(action=expected, direction=GrpcHIDEvent.UP)
                ),
            )

    def test_swipe(self) -> None:
        deltas = [None, 5]
        for delta in deltas:
            self.assertEqual(
                event_to_grpc(
                    HIDSwipe(start=Point(x=1, y=2), end=Point(x=3, y=4), delta=delta)
                ),
                GrpcHIDEvent(
                    swipe=GrpcHIDSwipe(
                        start=GrpcPoint(x=1, y=2), end=GrpcPoint(x=3, y=4), delta=delta
                    )
                ),
            )

    def test_delay(self) -> None:
        self.assertEqual(
            event_to_grpc(HIDDelay(duration=1)),
            GrpcHIDEvent(delay=GrpcHIDDelay(duration=1)),
        )



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

import asyncio
import functools
import inspect
import logging
import unittest
import unittest.mock as _mock
import warnings
from typing import Awaitable, Callable, TypeVar, cast

# pyre-ignore
from unittest.case import _Outcome


_RT = TypeVar("_RT")  # Return Generic
FuncType = Callable[..., Awaitable[_RT]]
_F = TypeVar("_F", bound=FuncType)  # Function Generic


def _tasks_warning(task_set):
    if task_set:
        warnings.warn(
            "There are tasks already on the event loop before running "
            f"the testmethod: {task_set}",
            stacklevel=0,
        )
        warnings.warn(
            "This may mean that something is creating tasks at import time",
            stacklevel=0,
        )


def awaitable(func: _F) -> _F:
    """
    What ever we are decorating, make it awaitable.
    This is not pretty, but useful when we don't know what
    we are accepting, like for unittests methods
    """

    @functools.wraps(func)
    async def new_func(*args, **kws):
        result = func(*args, **kws)
        if inspect.isawaitable(result):
            return await result
        return result

    return cast(_F, new_func)


class TestCase(unittest.TestCase):
    def __init__(self, methodName="runTest", loop=None):
        self.loop = loop or asyncio.new_event_loop()
        asyncio.set_event_loop(self.loop)
        self.loop.set_debug(True)
        super().__init__(methodName)

    async def run_async(self, testMethod, outcome, expecting_failure):
        with outcome.testPartExecutor(self):
            await awaitable(self.setUp)()
        if outcome.success:
            outcome.expecting_failure = expecting_failure
            with outcome.testPartExecutor(self, isTest=True):
                await awaitable(testMethod)()
            outcome.expecting_failure = False
            with outcome.testPartExecutor(self):
                await awaitable(self.tearDown)()
        await self.doCleanups()

    async def doCleanups(self):
        outcome = self._outcome or _Outcome()
        while self._cleanups:
            function, args, kwargs = self._cleanups.pop()
            with outcome.testPartExecutor(self):
                await awaitable(function)(*args, **kwargs)

    async def debug_async(self, testMethod):
        await awaitable(self.setUp)()
        await awaitable(testMethod)()
        await awaitable(self.tearDown)()
        while self._cleanups:
            function, args, kwargs = self._cleanups.pop(-1)
            await awaitable(function)(*args, **kwargs)

    @_mock.patch("asyncio.base_events.logger")
    @_mock.patch("asyncio.coroutines.logger")
    def asyncio_orchestration_debug(self, testMethod, b_log, c_log):
        asyncio.set_event_loop(self.loop)
        real_logger = logging.getLogger("asyncio").error
        c_log.error.side_effect = b_log.error.side_effect = real_logger
        # Don't make testmethods cleanup tasks that existed before them
        before_tasks = asyncio.Task.all_tasks(self.loop)
        _tasks_warning(before_tasks)
        debug_async = self.debug_async(testMethod)
        self.loop.run_until_complete(debug_async)

        if c_log.error.called or b_log.error.called:
            self.fail("asyncio logger.error() called!")
        # Sometimes we end up with a reference to our task for debug_async
        tasks = {
            t
            for t in asyncio.Task.all_tasks(self.loop) - before_tasks
            if not (t._coro == debug_async and t.done())
        }
        del before_tasks
        self.assertEqual(set(), tasks, "left over asyncio tasks!")

    @_mock.patch("asyncio.base_events.logger")
    @_mock.patch("asyncio.coroutines.logger")
    def asyncio_orchestration_outcome(
        self, testMethod, outcome, expecting_failure, b_log, c_log
    ):
        asyncio.set_event_loop(self.loop)
        real_logger = logging.getLogger("asyncio").error
        c_log.error.side_effect = b_log.error.side_effect = real_logger
        # Don't make testmethods cleanup tasks that existed before them
        before_tasks = asyncio.Task.all_tasks(self.loop)
        _tasks_warning(before_tasks)
        run_async = self.run_async(testMethod, outcome, expecting_failure)
        ignore_tasks = getattr(
            testMethod, "__unittest_asyncio_taskleaks__", False
        ) or getattr(self, "__unittest_asyncio_taskleaks__", False)
        with outcome.testPartExecutor(self):
            self.loop.run_until_complete(run_async)
            # Restore expecting_faiures so we can test the below
            outcome.expecting_failure = expecting_failure
            if c_log.error.called or b_log.error.called:
                self.fail("asyncio logger.error() called!")

            # Sometimes we end up with a reference to our task for run_async
            tasks = {
                t
                for t in asyncio.Task.all_tasks(self.loop) - before_tasks
                if not (t._coro == run_async and t.done())
            }
            del before_tasks
            if ignore_tasks and tasks:
                warnings.warn(
                    "There are left over asyncio tasks after running "
                    f"testmethod: {tasks}",
                    stacklevel=0,
                )
            else:
                self.assertEqual(set(), tasks, "left over asyncio tasks!")

    # pyre-ignore
    def run(self, result=None):
        """
        This is a complete copy of TestCase.run
        But with some asyncio worked into it.
        """
        orig_result = result
        if result is None:
            result = self.defaultTestResult()
            startTestRun = getattr(result, "startTestRun", None)
            if startTestRun is not None:
                startTestRun()

        result.startTest(self)

        testMethod = getattr(self, self._testMethodName)
        if getattr(self.__class__, "__unittest_skip__", False) or getattr(
            testMethod, "__unittest_skip__", False
        ):
            # If the class or method was skipped.
            try:
                skip_why = getattr(
                    self.__class__, "__unittest_skip_why__", ""
                ) or getattr(testMethod, "__unittest_skip_why__", "")
                self._addSkip(result, self, skip_why)  # noqa T484
            finally:
                result.stopTest(self)
            return None
        expecting_failure_method = getattr(
            testMethod, "__unittest_expecting_failure__", False
        )
        expecting_failure_class = getattr(self, "__unittest_expecting_failure__", False)
        expecting_failure = expecting_failure_class or expecting_failure_method
        outcome = _Outcome(result)
        try:
            self._outcome = outcome

            self.asyncio_orchestration_outcome(testMethod, outcome, expecting_failure)

            for test, reason in outcome.skipped:
                self._addSkip(result, test, reason)  # noqa T484
            self._feedErrorsToResult(result, outcome.errors)  # noqa T484
            if outcome.success:
                if expecting_failure:
                    if outcome.expectedFailure:
                        self._addExpectedFailure(  # noqa T484
                            result, outcome.expectedFailure
                        )
                    else:
                        self._addUnexpectedSuccess(result)  # noqa T484
                else:
                    result.addSuccess(self)
            return result
        finally:
            result.stopTest(self)
            if orig_result is None:
                stopTestRun = getattr(result, "stopTestRun", None)
                if stopTestRun is not None:
                    stopTestRun()

            # explicitly break reference cycles:
            # outcome.errors -> frame -> outcome -> outcome.errors
            # outcome.expectedFailure -> frame -> outcome -> outcome.expectedFailure
            outcome.errors.clear()
            outcome.expectedFailure = None

            # clear the outcome, no more needed
            self._outcome = None

    # pyre-ignore
    def debug(self):
        self.asyncio_orchestration_debug(getattr(self, self._testMethodName))


class AsyncMock(_mock.Mock):
    """ Mock subclass which can be awaited on. Use this as new_callable
    to patch calls on async functions. Can also be used as an async context
    manager - returns self.
    """

    def __call__(self, *args, **kwargs):
        sup = super(AsyncMock, self)

        async def coro():
            return sup.__call__(*args, **kwargs)

        return coro()

    def __await__(self):
        # Calling await on a Mock/AsyncMock object will result in
        # a TypeError. Instead, return the coroutine created above
        # to be awaited on
        return self().__await__()

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc, tb):
        pass


class AsyncContextManagerMock:
    """
    Helper mocking class to handle context manager consturcts.
    Example of usage:
    async def target():
        async with aiofiles.open('/tmp/b.txt') as f:
            return await f.read()

    class TestContextManager(TestCase):
            async def test_1(self):
                m = AsyncMock()
                m.read.return_value = 'fff'
                with async_mock.patch(
                    'aiofiles.open',
                    return_value=AsyncContextManagerMock(return_value=m)
                ):
                    r = await target()
                    self.assertEquals(r, 'fff')
    """

    def __init__(self, *args, **kwargs):
        self._mock = AsyncMock(*args, **kwargs)

    async def __aenter__(self, *args, **kwargs):
        return await self._mock(*args, **kwargs)

    async def __aexit__(self, exc_type, exc, tb):
        pass


def ignoreTaskLeaks(test_item):
    test_item.__unittest_asyncio_taskleaks__ = True
    return test_item



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

import json
from argparse import ArgumentParser, Namespace

from idb.cli.commands.base import TargetCommand
from idb.common.types import IdbClient


class AppInstallCommand(TargetCommand):
    @property
    def description(self) -> str:
        return "Install an application"

    @property
    def name(self) -> str:
        return "install"

    def add_parser_arguments(self, parser: ArgumentParser) -> None:
        parser.add_argument(
            "bundle_path", help="Path to the .app/.ipa to install", type=str
        )
        super().add_parser_arguments(parser)

    async def run_with_client(self, args: Namespace, client: IdbClient) -> None:
        artifact = await client.install(args.bundle_path)
        if args.json:
            print(
                json.dumps(
                    {"installedAppBundleId": artifact.name, "uuid": artifact.uuid}
                )
            )
        else:
            print(f"Installed: {artifact.name} {artifact.uuid}")


class AppUninstallCommand(TargetCommand):
    @property
    def description(self) -> str:
        return "Uninstall an application"

    @property
    def name(self) -> str:
        return "uninstall"

    def add_parser_arguments(self, parser: ArgumentParser) -> None:
        parser.add_argument(
            "bundle_id", help="Bundle ID of application to uninstall", type=str
        )
        super().add_parser_arguments(parser)

    async def run_with_client(self, args: Namespace, client: IdbClient) -> None:
        await client.uninstall(bundle_id=args.bundle_id)


class AppTerminateCommand(TargetCommand):
    @property
    def description(self) -> str:
        return "Terminate a running application"

    @property
    def name(self) -> str:
        return "terminate"

    def add_parser_arguments(self, parser: ArgumentParser) -> None:
        parser.add_argument("bundle_id", help="Bundle id of the app to kill", type=str)
        super().add_parser_arguments(parser)

    async def run_with_client(self, args: Namespace, client: IdbClient) -> None:
        await client.terminate(args.bundle_id)



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

import json
import os
from argparse import SUPPRESS, ArgumentParser, Namespace
from typing import Dict, Optional

from idb.cli.commands.base import BaseCommand
from idb.common.constants import DEFAULT_DAEMON_GRPC_PORT, DEFAULT_DAEMON_PORT
from idb.common.signal import signal_handler_event
from idb.common.types import IdbException


class DaemonCommand(BaseCommand):
    @property
    def description(self) -> str:
        return "This command is deprecated. the idb daemon is not used anymore."

    @property
    def name(self) -> str:
        return "daemon"

    def add_parser_arguments(self, parser: ArgumentParser) -> None:
        parser.add_argument(
            "--daemon-port", help=SUPPRESS, type=int, default=DEFAULT_DAEMON_PORT
        )
        parser.add_argument(
            "--reply-fd",
            help="File descriptor to write port the daemon was started on",
            type=int,
            required=False,
        )
        parser.add_argument(
            "--notifier-path",
            help="path to binary that notifies daemon about devices/simulators.",
            type=str,
            required=False,
        )
        parser.add_argument(
            "--daemon-grpc-port",
            help="Port for the daemon to listen to grpc on",
            type=int,
            default=DEFAULT_DAEMON_GRPC_PORT,
        )
        parser.add_argument(
            "--prefer-ipv6",
            help="If set, always return ipv6 port if available when --reply-fd is used",
            action="store_true",
            default=False,
        )
        super().add_parser_arguments(parser)

    async def _run_impl(self, args: Namespace) -> None:
        try:
            # leaving the daemon command with a dummy output
            # will remove after all uses are removed
            ports = {"ipv4_grpc_port": 0, "ipv6_grpc_port": 0}
            print(json.dumps(ports), sep="\n", flush=True)
            self._reply_with_port(args.reply_fd, args.prefer_ipv6, ports)
            await signal_handler_event("server").wait()
        except IdbException as ex:
            self.logger.exception("Exception in main")
            raise ex
        finally:
            self.logger.info("Exiting")

    def _reply_with_port(
        self, reply_fd: Optional[int], prefer_ipv6: bool, ports: Dict[str, int]
    ) -> None:
        if not reply_fd:
            return

        prefix = "ipv6_" if prefer_ipv6 else "ipv4_"
        all_ports = {
            key.split(prefix)[1]: value
            for (key, value) in ports.items()
            if key.startswith(prefix)
        }
        self.logger.info(f"Replying to fd {reply_fd} with {all_ports}")
        with os.fdopen(reply_fd, "w") as f:
            f.write(json.dumps(all_ports) + "\n")



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

import os
from typing import Dict


def get_env_with_idb_prefix() -> Dict[str, str]:
    env = dict(os.environ)
    env = {key: env[key] for key in env if key.startswith("IDB_")}
    return {key[len("IDB_") :]: env[key] for key in env}



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

from argparse import ArgumentParser, Namespace

from idb.cli.commands.base import TargetCommand
from idb.common.types import IdbClient


class ContactsUpdateCommand(TargetCommand):
    @property
    def description(self) -> str:
        return "Updates the contacts"

    @property
    def name(self) -> str:
        return "update"

    def add_parser_arguments(self, parser: ArgumentParser) -> None:
        parser.add_argument(
            "contacts_path",
            help="Path to the directory containing contacts sqlite databases",
            type=str,
        )
        super().add_parser_arguments(parser)

    async def run_with_client(self, args: Namespace, client: IdbClient) -> None:
        await client.contacts_update(contacts_path=args.contacts_path)



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

import os.path
import plistlib
import tempfile
from unittest import TestCase

from idb.common.xctest import extract_paths_from_xctestrun


class XCTestsTestCase(TestCase):
    async def test_extract_paths_from_xctestrun(self) -> None:
        with tempfile.TemporaryDirectory() as tmp_dir:
            file_path = os.path.join(tmp_dir, "test.plist")
            with open(file_path, "wb+") as f:
                plistlib.dump(
                    {
                        "key1": {"TestHostPath": "__TESTROOT__/rest1"},
                        "key2": {"TestHostPath": "__TESTROOT__/rest2"},
                    },
                    f,
                )
            results = extract_paths_from_xctestrun(file_path)
            self.assertEqual(
                [file_path, tmp_dir + "/rest1", tmp_dir + "/rest2"], results
            )



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

import asyncio
import signal
from sys import stderr


def signal_handler_event(name: str) -> asyncio.Event:
    loop = asyncio.get_event_loop()
    stop = asyncio.Event()

    def signal_handler(sig: signal.Signals) -> None:
        print(f"\nStopping {name}", file=stderr)
        stop.set()

    for sig in [signal.SIGTERM, signal.SIGINT]:
        loop.add_signal_handler(sig, lambda: signal_handler(sig))

    print(f"Running {name} until ^C", file=stderr)
    return stop



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

from argparse import ArgumentParser, Namespace

from idb.cli.commands.base import TargetCommand
from idb.common.types import IdbClient


class SetLocationCommand(TargetCommand):
    @property
    def description(self) -> str:
        return "Set a simulator's location"

    @property
    def name(self) -> str:
        return "set-location"

    def add_parser_arguments(self, parser: ArgumentParser) -> None:
        parser.add_argument("latitude", help="Latitude to set", type=float)
        parser.add_argument("longitude", help="Longitude to set", type=float)
        super().add_parser_arguments(parser)

    async def run_with_client(self, args: Namespace, client: IdbClient) -> None:
        await client.set_location(args.latitude, args.longitude)



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.


from logging import Logger
from typing import NamedTuple, Optional

from idb.grpc.idb_grpc import CompanionServiceStub


class CompanionClient(NamedTuple):
    stub: CompanionServiceStub
    is_local: bool
    udid: Optional[str]
    logger: Logger
    is_companion_available: bool = False



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

from argparse import ArgumentParser, Namespace

from idb.cli.commands.base import TargetCommand
from idb.common.types import IdbClient


class DebugServerStartCommand(TargetCommand):
    @property
    def description(self) -> str:
        return "Start the Debug Server"

    @property
    def name(self) -> str:
        return "start"

    def add_parser_arguments(self, parser: ArgumentParser) -> None:
        super().add_parser_arguments(parser)
        parser.add_argument("bundle_id", help="The bundle id to debug")

    async def run_with_client(self, args: Namespace, client: IdbClient) -> None:
        commands = await client.debugserver_start(bundle_id=args.bundle_id)
        print(*commands, sep="\n")


class DebugServerStopCommand(TargetCommand):
    @property
    def description(self) -> str:
        return "Stop the debug server"

    @property
    def name(self) -> str:
        return "stop"

    async def run_with_client(self, args: Namespace, client: IdbClient) -> None:
        await client.debugserver_stop()


class DebugServerStatusCommand(TargetCommand):
    @property
    def description(self) -> str:
        return "Get the status of the debug server"

    @property
    def name(self) -> str:
        return "status"

    async def run_with_client(self, args: Namespace, client: IdbClient) -> None:
        commands = await client.debugserver_status()
        if commands is None:
            print("Not Running")
        else:
            print(*commands, sep="\n")



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

from argparse import ArgumentParser, Namespace

from idb.cli.commands.base import ConnectingCommand
from idb.common.format import human_format_target_info, json_format_target_info
from idb.common.types import IdbClient


class ListTargetsCommand(ConnectingCommand):
    @property
    def description(self) -> str:
        return "List the connected targets"

    @property
    def name(self) -> str:
        return "list-targets"

    def add_parser_arguments(self, parser: ArgumentParser) -> None:
        super().add_parser_arguments(parser)

    async def run_with_client(self, args: Namespace, client: IdbClient) -> None:
        targets = await client.list_targets()
        if len(targets) == 0:
            if not args.json:
                print("No available targets")
            return

        formatter = human_format_target_info
        if args.json:
            formatter = json_format_target_info
        for target in targets:
            print(formatter(target))



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

from argparse import Namespace

from idb.cli.commands.base import TargetCommand
from idb.common.format import (
    human_format_installed_app_info,
    json_format_installed_app_info,
)
from idb.common.types import IdbClient


class ListAppsCommand(TargetCommand):
    @property
    def description(self) -> str:
        return "List the installed apps"

    @property
    def name(self) -> str:
        return "list-apps"

    async def run_with_client(self, args: Namespace, client: IdbClient) -> None:
        apps = await client.list_apps()
        formatter = human_format_installed_app_info
        if args.json:
            formatter = json_format_installed_app_info
        for app in apps:
            print(formatter(app))



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

from idb.common.types import ScreenDimensions, TargetDescription
from idb.grpc.idb_pb2 import (
    ScreenDimensions as GrpcScreenDimensions,
    TargetDescription as GrpcTargetDescription,
)
from idb.ipc.mapping.companion import companion_to_grpc, companion_to_py


def target_to_grpc(target: TargetDescription) -> GrpcTargetDescription:
    return GrpcTargetDescription(
        udid=target.udid,
        name=target.name,
        screen_dimensions=screen_dimensions_to_grpc(target.screen_dimensions)
        if target.screen_dimensions
        else None,
        state=target.state,
        target_type=target.target_type,
        os_version=target.os_version,
        architecture=target.architecture,
        companion_info=companion_to_grpc(target.companion_info)
        if target.companion_info
        else None,
        daemon_host=None,
        daemon_port=None,
    )


def target_to_py(target: GrpcTargetDescription) -> TargetDescription:
    return TargetDescription(
        udid=target.udid,
        name=target.name,
        screen_dimensions=screen_dimensions_to_py(target.screen_dimensions)
        if target.screen_dimensions
        else None,
        state=target.state,
        target_type=target.target_type,
        os_version=target.os_version,
        architecture=target.architecture,
        companion_info=companion_to_py(target.companion_info)
        if target.companion_info
        else None,
    )


def screen_dimensions_to_grpc(dimensions: ScreenDimensions) -> GrpcScreenDimensions:
    return GrpcScreenDimensions(
        width=dimensions.width,
        height=dimensions.height,
        density=dimensions.density,
        width_points=dimensions.width_points,
        height_points=dimensions.height_points,
    )


def screen_dimensions_to_py(dimensions: GrpcScreenDimensions) -> ScreenDimensions:
    return ScreenDimensions(
        width=dimensions.width,
        height=dimensions.height,
        density=dimensions.density,
        width_points=dimensions.width_points,
        height_points=dimensions.height_points,
    )



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

from argparse import REMAINDER, ArgumentParser, Namespace
from typing import List, Optional

from idb.cli.commands.base import TargetCommand
from idb.common.signal import signal_handler_event
from idb.common.types import IdbClient


class LogCommand(TargetCommand):
    @property
    def description(self) -> str:
        return "Obtain logs from the target"

    @property
    def name(self) -> str:
        return "log"

    def add_parser_arguments(self, parser: ArgumentParser) -> None:
        parser.add_argument(
            "log_arguments",
            help="""\
Example: idb log -- --style json
Possible arguments:
--system | --process (pid|process) | --parent (pid|process) ]
    [ --level default|info|debug][ --predicate <predicate> ]
    [ --source ][ --style (syslog|json) ]
    [ --timeout <num>[m|h|d] ][ --type activity|log|trace ]

Examples:
log stream --level=info
log stream --predicate examples:
    --predicate 'eventMessage contains "my message"'
    --predicate 'eventType == logEvent and messageType == info'
    --predicate 'processImagePath endswith "d"'
    --predicate 'not processImagePath contains[c] "some spammer"'
    --predicate 'processID < 100'
    --predicate 'senderImagePath beginswith "my sender"'""",
            default=[],
            nargs=REMAINDER,
        )
        super().add_parser_arguments(parser)

    async def run_with_client(self, args: Namespace, client: IdbClient) -> None:
        async for chunk in client.tail_logs(
            stop=signal_handler_event("log"),
            arguments=self.normalise_log_arguments(args.log_arguments),
        ):
            print(chunk, end="")
        print("")

    def normalise_log_arguments(
        self, log_arguments: Optional[List[str]]
    ) -> Optional[List[str]]:
        if log_arguments is None:
            return None

        if len(log_arguments) > 0 and log_arguments[0] == "--":
            log_arguments = log_arguments[1:]

        return log_arguments


class CompanionLogCommand(TargetCommand):
    @property
    def description(self) -> str:
        return "Obtain logs from the companion"

    @property
    def name(self) -> str:
        return "log"

    async def run_with_client(self, args: Namespace, client: IdbClient) -> None:
        async for chunk in client.tail_companion_logs(stop=signal_handler_event("log")):
            print(chunk, end="")
        print("")



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

from idb.common.networking import _get_ports
from idb.utils.testing import TestCase, ignoreTaskLeaks


@ignoreTaskLeaks
class NetworkingTests(TestCase):
    def test_get_ports(self) -> None:
        data = '{"grpc_port": 1235}'
        ports = _get_ports(data)
        self.assertEqual(ports, (1235))



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.
import re


SIMULATOR_UDID = r"^[0-9A-F]{8}-[0-9A-F]{4}-[0-9A-F]{4}-[0-9A-F]{4}-[0-9A-F]{12}$"
OLD_DEVICE_UDID = r"^[0-9a-f]{40}$"
NEW_DEVICE_UDID = r"^[0-9]{8}-[0-9A-F]{16}$"
UDID = fr"({SIMULATOR_UDID}|{OLD_DEVICE_UDID}|{NEW_DEVICE_UDID})"


def is_udid(udid: str) -> bool:
    return bool(re.match(UDID, udid))



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

from typing import AsyncIterator

from idb.grpc.idb_pb2 import RecordResponse


async def generate_video_bytes(
    stream: AsyncIterator[RecordResponse],
) -> AsyncIterator[bytes]:
    async for response in stream:
        data = response.payload.data
        yield data



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

import json
from argparse import SUPPRESS, ArgumentParser, Namespace
from typing import Union

import idb.common.plugin as plugin
from idb.cli.commands.base import ConnectingCommand
from idb.common.types import Address, IdbClient, IdbException
from idb.common.udid import is_udid


def get_destination(args: Namespace) -> Union[Address, str]:
    if is_udid(args.companion):
        return args.companion
    elif args.port and args.companion:
        return Address(host=args.companion, port=args.port)
    else:
        raise ConnectCommandException(
            "provide either a UDID or the host and port of the companion"
        )


class ConnectCommandException(Exception):
    pass


class ConnectCommand(ConnectingCommand):
    @property
    def description(self) -> str:
        return "Connect to a companion"

    @property
    def name(self) -> str:
        return "connect"

    def add_parser_arguments(self, parser: ArgumentParser) -> None:
        parser.add_argument(
            "companion",
            help="Host the companion is running on. or the UDID of the target",
            type=str,
        )
        parser.add_argument(
            "port",
            help="Port the companion is running on",
            type=int,
            nargs="?",
            default=None,
        )
        # not used and suppressed. remove after the removal of thrift is deployed everywhere
        parser.add_argument(
            "grpc_port", help=SUPPRESS, type=int, nargs="?", default=None
        )
        super().add_parser_arguments(parser)

    async def run_with_client(self, args: Namespace, client: IdbClient) -> None:
        try:
            destination = get_destination(args=args)
            connect_response = await client.connect(
                destination=destination,
                metadata={
                    key: value
                    for (key, value) in plugin.resolve_metadata(self.logger).items()
                    if isinstance(value, str)
                },
            )
            if connect_response:
                if args.json:
                    print(
                        json.dumps(
                            {
                                "udid": connect_response.udid,
                                "is_local": connect_response.is_local,
                            }
                        )
                    )
                else:
                    print(
                        f"udid: {connect_response.udid} is_local: {connect_response.is_local}"
                    )

        except IdbException:
            raise ConnectCommandException(
                f"""Could not connect to {args.companion:}:{args.port}.
            Make sure both host and port are correct and reachable"""
            )



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

import logging
import os
from abc import ABCMeta, abstractmethod
from argparse import ArgumentParser, Namespace
from typing import Dict, List, Optional

import idb.common.plugin as plugin
from idb.client.grpc import GrpcClient
from idb.common.constants import DEFAULT_DAEMON_GRPC_PORT, DEFAULT_DAEMON_HOST
from idb.common.logging import log_call
from idb.common.types import IdbClient


class Command(metaclass=ABCMeta):
    @property
    @abstractmethod
    def description(self) -> str:
        raise Exception("subclass")

    @property
    @abstractmethod
    def name(self) -> str:
        raise Exception("subclass")

    @property
    def aliases(self) -> List[str]:
        return []

    @abstractmethod
    def add_parser_arguments(self, parser: ArgumentParser) -> None:
        raise Exception("subclass")

    @abstractmethod
    async def run(self, args: Namespace) -> None:
        raise Exception("subclass")

    @property
    def allow_unknown_args(self) -> bool:
        return False


class CompositeCommand(Command, metaclass=ABCMeta):
    parser: Optional[ArgumentParser] = None

    @property
    @abstractmethod
    def subcommands(self) -> List[Command]:
        pass

    @property
    def subcommands_by_name(self) -> Dict[str, Command]:
        def add_unique_cmd(aDict: Dict[str, Command], key: str, value: Command) -> None:
            assert key not in aDict, f'Subcommand by name "{key}" already exists'
            aDict[key] = value

        aDict: Optional[Dict[str, Command]] = getattr(
            self, "_subcommands_by_name", None
        )
        if aDict is None:
            aDict = {}
            # pyre-fixme[16]: `CompositeCommand` has no attribute
            #  `_subcommands_by_name`.
            self._subcommands_by_name = aDict
            for cmd in self.subcommands:
                add_unique_cmd(aDict, cmd.name, cmd)
                for alias in cmd.aliases:
                    add_unique_cmd(aDict, alias, cmd)

        return aDict

    def add_parser_arguments(self, parser: ArgumentParser) -> None:
        self.parser = parser
        sub_parsers = parser.add_subparsers(dest=self.name)
        for command in self.subcommands:
            sub_parser = sub_parsers.add_parser(
                command.name, help=command.description, aliases=command.aliases
            )
            command.add_parser_arguments(sub_parser)

    def _get_subcommand_for_args(self, args: Namespace) -> Command:
        subcmd_name = getattr(args, self.name)
        if self.parser and subcmd_name is None:
            self.parser.print_help()
            # This terminates the program with exit code 2
            self.parser.error(f"No subcommand found for {self.name}")
        subcmd = self.subcommands_by_name[subcmd_name]
        assert subcmd is not None, "subcommand %r doesn't exist" % subcmd_name
        return subcmd

    async def run(self, args: Namespace) -> None:
        return await self._get_subcommand_for_args(args).run(args)


class BaseCommand(Command, metaclass=ABCMeta):
    def __init__(self) -> None:
        super().__init__()
        # Will inherit log levels when the log level is set on the base logger in run()
        self.logger: logging.Logger = logging.getLogger(self.name)

    def add_parser_arguments(self, parser: ArgumentParser) -> None:
        parser.add_argument(
            "--log",
            dest="log_level",
            choices=["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"],
            default="WARNING",
            help="Set the logging level",
        )
        parser.add_argument(
            "--json",
            action="store_true",
            default=False,
            help="Create json structured output",
        )

    async def run(self, args: Namespace) -> None:
        # Set the log level on the base logger
        logging.getLogger().setLevel(args.log_level)
        name = self.__class__.__name__
        self.logger.debug(f"{name} command run with: {args}")
        async with log_call(
            name=name, metadata=plugin.resolve_metadata(logger=self.logger)
        ):
            await self._run_impl(args)

    @abstractmethod
    async def _run_impl(self, args: Namespace) -> None:
        raise Exception("subclass")


# pyre-fixme[44]: `ConnectingCommand` non-abstract class with abstract methods
class ConnectingCommand(BaseCommand):
    def add_parser_arguments(self, parser: ArgumentParser) -> None:
        plugin.on_connecting_parser(parser=parser, logger=self.logger)
        parser.add_argument(
            "--daemon-host",
            help="Host the daemon is running on",
            type=str,
            default=DEFAULT_DAEMON_HOST,
        )
        parser.add_argument(
            "--force", help="Kill any implicitly running daemons", action="store_true"
        )
        parser.add_argument(
            "--daemon-grpc-port",
            help="Port the daemon is running it's grpc interface on",
            type=int,
            default=DEFAULT_DAEMON_GRPC_PORT,
        )
        super().add_parser_arguments(parser)

    async def _run_impl(self, args: Namespace) -> None:
        udid = vars(args).get("udid")
        client = GrpcClient(target_udid=udid, logger=self.logger)
        await self.run_with_client(args=args, client=client)

    @abstractmethod
    async def run_with_client(self, args: Namespace, client: IdbClient) -> None:
        pass


class TargetCommand(ConnectingCommand):
    def add_parser_arguments(self, parser: ArgumentParser) -> None:
        parser.add_argument(
            "--udid",
            help="Udid of target, can also be set with the IDB_UDID env var",
            default=os.environ.get("IDB_UDID"),
        )
        super().add_parser_arguments(parser)


class CommandGroup(CompositeCommand):
    def __init__(self, name: str, description: str, commands: List[Command]) -> None:
        self.commands = commands
        self._name = name
        self._description = description

    @property
    def name(self) -> str:
        return self._name

    @property
    def description(self) -> str:
        return self._description

    @property
    def subcommands(self) -> List[Command]:
        return self.commands



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

from argparse import ArgumentParser, Namespace

from idb.cli.commands.base import TargetCommand
from idb.common.types import HIDButtonType, IdbClient


class TapCommand(TargetCommand):
    @property
    def description(self) -> str:
        return "Tap On the Screen"

    @property
    def name(self) -> str:
        return "tap"

    def add_parser_arguments(self, parser: ArgumentParser) -> None:
        parser.add_argument("x", help="The x-coordinate", type=int)
        parser.add_argument("y", help="The y-coordinate", type=int)
        parser.add_argument("--duration", help="Press duration", type=float)
        super().add_parser_arguments(parser)

    async def run_with_client(self, args: Namespace, client: IdbClient) -> None:
        await client.tap(x=args.x, y=args.y, duration=args.duration)


class ButtonCommand(TargetCommand):
    @property
    def description(self) -> str:
        return "A single press of a button"

    @property
    def name(self) -> str:
        return "button"

    def add_parser_arguments(self, parser: ArgumentParser) -> None:
        parser.add_argument(
            "button",
            help="The button name",
            choices=[button.name for button in HIDButtonType],
            type=str,
        )
        parser.add_argument("--duration", help="Press duration", type=float)
        super().add_parser_arguments(parser)

    async def run_with_client(self, args: Namespace, client: IdbClient) -> None:
        await client.button(
            button_type=HIDButtonType[args.button], duration=args.duration
        )


class KeyCommand(TargetCommand):
    @property
    def description(self) -> str:
        return "A short press of a keycode"

    @property
    def name(self) -> str:
        return "key"

    def add_parser_arguments(self, parser: ArgumentParser) -> None:
        parser.add_argument("key", help="The key code", type=int)
        parser.add_argument("--duration", help="Press duration", type=float)
        super().add_parser_arguments(parser)

    async def run_with_client(self, args: Namespace, client: IdbClient) -> None:
        await client.key(keycode=args.key, duration=args.duration)


class KeySequenceCommand(TargetCommand):
    @property
    def description(self) -> str:
        return "A sequence of short presses of a keycode"

    @property
    def name(self) -> str:
        return "key-sequence"

    def add_parser_arguments(self, parser: ArgumentParser) -> None:
        parser.add_argument(
            "key_sequence",
            help="list of space separated key codes string (i.e. 1 2 3))",
            nargs="*",
        )
        super().add_parser_arguments(parser)

    async def run_with_client(self, args: Namespace, client: IdbClient) -> None:
        await client.key_sequence(key_sequence=list(map(int, args.key_sequence)))


class TextCommand(TargetCommand):
    @property
    def description(self) -> str:
        return "Input text"

    @property
    def name(self) -> str:
        return "text"

    def add_parser_arguments(self, parser: ArgumentParser) -> None:
        parser.add_argument("text", help="Text to input", type=str)
        super().add_parser_arguments(parser)

    async def run_with_client(self, args: Namespace, client: IdbClient) -> None:
        await client.text(text=args.text)


class SwipeCommand(TargetCommand):
    @property
    def description(self) -> str:
        return "Swipe from one point to another point"

    @property
    def name(self) -> str:
        return "swipe"

    def add_parser_arguments(self, parser: ArgumentParser) -> None:
        parser.add_argument(
            "x_start", help="The x-coordinate of the swipe start point", type=int
        )
        parser.add_argument(
            "y_start", help="The y-coordinate of the swipe start point", type=int
        )
        parser.add_argument(
            "x_end", help="The x-coordinate of the swipe end point", type=int
        )
        parser.add_argument(
            "y_end", help="The y-coordinate of the swipe end point", type=int
        )

        parser.add_argument(
            "--delta",
            dest="delta",
            help="delta in pixels between every touch point on the line "
            "between start and end points",
            type=int,
            required=False,
        )
        super().add_parser_arguments(parser)

    async def run_with_client(self, args: Namespace, client: IdbClient) -> None:
        await client.swipe(
            p_start=(args.x_start, args.y_start),
            p_end=(args.x_end, args.y_end),
            delta=args.delta,
        )



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

from typing import Optional, TypeVar


T = TypeVar("T")


def none_throws(optional: Optional[T]) -> T:
    assert optional is not None, "Unexpected None"
    return optional



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

import functools
import inspect
import logging
import time
from concurrent.futures import CancelledError
from types import TracebackType
from typing import Any, AsyncContextManager, Collection, Optional, Tuple, Type

import idb.common.plugin as plugin
from grpclib.const import Status
from grpclib.exceptions import GRPCError
from idb.common.types import LoggingMetadata
from idb.utils.typing import none_throws


logger: logging.Logger = logging.getLogger("idb")


def _initial_info(
    args: Collection[Any], metadata: Optional[LoggingMetadata]  # pyre-ignore
) -> Tuple[LoggingMetadata, int]:
    _metadata: LoggingMetadata = metadata or {}
    if len(args):
        # pyre-fixme[16]: `Collection` has no attribute `__getitem__`.
        self_meta: Optional[LoggingMetadata] = getattr(args[0], "metadata", None)
        if self_meta:
            _metadata.update(self_meta)
    start = int(time.time())
    return (_metadata, start)


class log_call(AsyncContextManager[None]):
    def __init__(
        self,
        name: Optional[str] = None,
        metadata: Optional[LoggingMetadata] = None,
        translate_exceptions: bool = False,
    ) -> None:
        self.name = name
        self.metadata: LoggingMetadata = metadata or {}
        self.start: Optional[int] = None
        self.translate_exceptions = translate_exceptions

    async def __aenter__(self) -> None:
        name = none_throws(self.name)
        logger.debug(f"{self.name} called")
        self.start = int(time.time())
        await plugin.before_invocation(name=name, metadata=self.metadata)

    async def __aexit__(
        self,
        exc_type: Optional[Type[Exception]],
        exception: Optional[Exception],
        traceback: Optional[TracebackType],
    ) -> bool:
        name = none_throws(self.name)
        duration = int((time.time() - none_throws(self.start)) * 1000)
        if exception:
            logger.debug(f"{name} failed")
            await plugin.failed_invocation(
                name=name,
                duration=duration,
                exception=exception,
                metadata=self.metadata,
            )
        else:
            logger.debug(f"{name} succeeded")
            await plugin.after_invocation(
                name=name, duration=duration, metadata=self.metadata
            )
        return False

    def __call__(self, function) -> Any:  # pyre-ignore
        _name = self.name or function.__name__

        @functools.wraps(function)
        async def _async_wrapper(*args: Any, **kwargs: Any) -> Any:  # pyre-ignore
            logger.debug(f"{_name} called")
            (_metadata, start) = _initial_info(args, self.metadata)
            await plugin.before_invocation(name=_name, metadata=_metadata)
            try:
                value = await function(*args, **kwargs)
                logger.debug(f"{_name} succeeded")
                await plugin.after_invocation(
                    name=_name,
                    duration=int((time.time() - start) * 1000),
                    metadata=_metadata,
                )
                return value
            except CancelledError as ex:
                logger.debug(f"{_name} cancelled")
                _metadata["cancelled"] = True
                await plugin.after_invocation(
                    name=_name,
                    duration=int((time.time() - start) * 1000),
                    metadata=_metadata,
                )
                raise self.translate_exception(ex)
            except Exception as ex:
                logger.debug(f"{_name} failed")
                await plugin.failed_invocation(
                    name=_name,
                    duration=int((time.time() - start) * 1000),
                    exception=ex,
                    metadata=_metadata,
                )
                raise self.translate_exception(ex)

        @functools.wraps(function)
        async def _async_gen_wrapper(*args, **kwargs) -> Any:  # pyre-ignore
            logger.debug(f"{_name} started")
            (_metadata, start) = _initial_info(args, self.metadata)
            await plugin.before_invocation(name=_name, metadata=_metadata)
            try:
                async for value in function(*args, **kwargs):
                    yield value
                logger.debug(f"{_name} finished")
                await plugin.after_invocation(
                    name=_name,
                    duration=int((time.time() - start) * 1000),
                    metadata=_metadata,
                )
            except CancelledError as ex:
                logger.debug(f"{_name} cancelled")
                _metadata["cancelled"] = True
                await plugin.after_invocation(
                    name=_name,
                    duration=int((time.time() - start) * 1000),
                    metadata=_metadata,
                )
                raise self.translate_exception(ex)
            except Exception as ex:
                logger.debug(f"{_name} failed")
                await plugin.failed_invocation(
                    name=_name,
                    duration=int((time.time() - start) * 1000),
                    exception=ex,
                    metadata=_metadata,
                )
                raise self.translate_exception(ex)

        if inspect.isasyncgenfunction(function):
            return _async_gen_wrapper
        else:
            return _async_wrapper

    def translate_exception(self, exception: Exception) -> Exception:
        if self.translate_exceptions and not isinstance(exception, GRPCError):
            return GRPCError(Status.INTERNAL, exception.args[0])
        return exception



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

import json
from argparse import ArgumentParser, Namespace

from idb.cli.commands.base import TargetCommand
from idb.common.types import IdbClient


class DylibInstallCommand(TargetCommand):
    @property
    def description(self) -> str:
        return "Install an dylib"

    @property
    def name(self) -> str:
        return "install"

    def add_parser_arguments(self, parser: ArgumentParser) -> None:
        parser.add_argument("dylib_path", help="Path to the dylib to install", type=str)
        super().add_parser_arguments(parser)

    async def run_with_client(self, args: Namespace, client: IdbClient) -> None:
        artifact = await client.install_dylib(args.dylib_path)
        if args.json:
            print(json.dumps({"dylib": artifact.name, "uuid": artifact.uuid}))
        else:
            print(f"Installed: {artifact.name} {artifact.uuid}")



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

import json
import os
from abc import abstractmethod
from argparse import ArgumentParser, Namespace
from typing import Any, List, NamedTuple, Optional, Tuple

from idb.cli.commands.base import TargetCommand
from idb.common.types import IdbClient


class NoBundleIdentifierProvidedException(BaseException):
    pass


class BundleWithPath(NamedTuple):
    bundle_id: Optional[str]
    path: str

    @classmethod
    def parse(cls, argument: str) -> "BundleWithPath":
        split = argument.split(sep=":", maxsplit=1)
        if len(split) == 1:
            return BundleWithPath(bundle_id=None, path=split[0])
        return BundleWithPath(bundle_id=split[0], path=split[1])


def _extract_bundle_id(args: Namespace) -> str:
    if args.bundle_id:
        return args.bundle_id
    values = []
    for value in vars(args).values():
        if isinstance(value, List):
            values.extend(value)
        else:
            values.append(value)
    for value in values:
        if not isinstance(value, BundleWithPath):
            continue
        bundle_id = value.bundle_id
        if bundle_id is None:
            continue
        # pyre-fixme[16]: `Namespace` has no attribute `bundle_id`.
        args.bundle_id = bundle_id
        return args.bundle_id
    raise NoBundleIdentifierProvidedException(f"No Bundle ID Provided in args {args}")


def _convert_args(args: Namespace) -> Tuple[Namespace, str]:
    def convert_value(value: Any) -> Any:  # pyre-ignore
        if isinstance(value, List):
            # pyre-fixme[18]: Global name `convert_value` is undefined.
            return [convert_value(x) for x in value]
        return value.path if isinstance(value, BundleWithPath) else value

    bundle_id = _extract_bundle_id(args)
    args = Namespace(
        **{
            key: convert_value(value)
            for (key, value) in vars(args).items()
            if key != "bundle_id"
        }
    )
    return (args, bundle_id)


# pyre-fixme[44]: `FSCommand` non-abstract class with abstract methods
class FSCommand(TargetCommand):
    def add_parser_arguments(self, parser: ArgumentParser) -> None:
        parser.add_argument(
            "--bundle-id",
            help="Bundle ID of application. Must be provi",
            type=str,
            required=False,
            default=None,
        )
        super().add_parser_arguments(parser)

    @abstractmethod
    async def run_with_bundle(
        self, bundle_id: str, args: Namespace, client: IdbClient
    ) -> None:
        pass

    async def run_with_client(self, args: Namespace, client: IdbClient) -> None:
        (args, bundle_id) = _convert_args(args)
        return await self.run_with_bundle(bundle_id=bundle_id, args=args, client=client)


class FSListCommand(FSCommand):
    @property
    def description(self) -> str:
        return "List a path inside an application's container"

    @property
    def name(self) -> str:
        return "list"

    @property
    def aliases(self) -> List[str]:
        return ["ls"]

    def add_parser_arguments(self, parser: ArgumentParser) -> None:
        parser.add_argument(
            "path", help="Source path", default="./", type=BundleWithPath.parse
        )
        super().add_parser_arguments(parser)

    async def run_with_bundle(
        self, bundle_id: str, args: Namespace, client: IdbClient
    ) -> None:
        paths = await client.ls(bundle_id=bundle_id, path=args.path)
        if args.json:
            print(json.dumps([{"path": item.path} for item in paths]))
        else:
            for item in paths:
                print(item.path)


class FSMkdirCommand(FSCommand):
    @property
    def description(self) -> str:
        return "Make a directory inside an application's container"

    @property
    def name(self) -> str:
        return "mkdir"

    def add_parser_arguments(self, parser: ArgumentParser) -> None:
        super().add_parser_arguments(parser)
        parser.add_argument(
            "path", help="Path to directory to create", type=BundleWithPath.parse
        )

    async def run_with_bundle(
        self, bundle_id: str, args: Namespace, client: IdbClient
    ) -> None:
        await client.mkdir(bundle_id=bundle_id, path=args.path)


class FSMoveCommand(FSCommand):
    @property
    def description(self) -> str:
        return "Move a path inside an application's container"

    @property
    def name(self) -> str:
        return "move"

    @property
    def aliases(self) -> List[str]:
        return ["mv"]

    def add_parser_arguments(self, parser: ArgumentParser) -> None:
        parser.add_argument(
            "src",
            help="Source paths relative to Container",
            nargs="+",
            type=BundleWithPath.parse,
        )
        parser.add_argument(
            "dst",
            help="Destination path relative to Container",
            type=BundleWithPath.parse,
        )
        super().add_parser_arguments(parser)

    async def run_with_bundle(
        self, bundle_id: str, args: Namespace, client: IdbClient
    ) -> None:
        await client.mv(bundle_id=bundle_id, src_paths=args.src, dest_path=args.dst)


class FSRemoveCommand(FSCommand):
    @property
    def description(self) -> str:
        return "Remove an item inside a container"

    @property
    def name(self) -> str:
        return "remove"

    @property
    def aliases(self) -> List[str]:
        return ["rm"]

    def add_parser_arguments(self, parser: ArgumentParser) -> None:
        parser.add_argument(
            "path",
            help="Path of item to remove (A directory will be recursively deleted)",
            nargs="+",
            type=BundleWithPath.parse,
        )
        super().add_parser_arguments(parser)

    async def run_with_bundle(
        self, bundle_id: str, args: Namespace, client: IdbClient
    ) -> None:
        await client.rm(bundle_id=bundle_id, paths=args.path)


class FSPushCommand(FSCommand):
    @property
    def description(self) -> str:
        return "Copy file(s) from local machine to target"

    @property
    def name(self) -> str:
        return "push"

    def add_parser_arguments(self, parser: ArgumentParser) -> None:
        parser.add_argument(
            "src_paths", help="Path of file(s) to copy to the target", nargs="+"
        )
        parser.add_argument(
            "dest_path",
            help=(
                "Directory relative to the data container of the application\n"
                "to copy the files into. Will be created if non-existent"
            ),
            type=BundleWithPath.parse,
        )
        super().add_parser_arguments(parser)

    async def run_with_bundle(
        self, bundle_id: str, args: Namespace, client: IdbClient
    ) -> None:
        return await client.push(
            bundle_id=bundle_id,
            src_paths=[os.path.abspath(path) for path in args.src_paths],
            dest_path=args.dest_path,
        )


class FSPullCommand(FSCommand):
    @property
    def description(self) -> str:
        return "Copy a file inside an application's container"

    @property
    def name(self) -> str:
        return "pull"

    def add_parser_arguments(self, parser: ArgumentParser) -> None:
        parser.add_argument(
            "src", help="Relative Container source path", type=BundleWithPath.parse
        )
        parser.add_argument("dst", help="Local destination path", type=str)
        super().add_parser_arguments(parser)

    async def run_with_bundle(
        self, bundle_id: str, args: Namespace, client: IdbClient
    ) -> None:
        await client.pull(
            bundle_id=bundle_id, src_path=args.src, dest_path=os.path.abspath(args.dst)
        )


class DeprecatedPushCommand(TargetCommand):
    @property
    def description(self) -> str:
        return "Copy file(s) from local machine to target"

    @property
    def name(self) -> str:
        return "push"

    def add_parser_arguments(self, parser: ArgumentParser) -> None:
        parser.add_argument(
            "src_paths", help="Path of file(s) to copy to the target", nargs="+"
        )
        parser.add_argument(
            "bundle_id", help="Bundle id of the app to contain these files", type=str
        )
        parser.add_argument(
            "dest_path",
            help=(
                "Directory relative to the data container of the application\n"
                "to copy the files into. Will be created if non-existent"
            ),
            type=str,
        )
        super().add_parser_arguments(parser)

    async def run_with_client(self, args: Namespace, client: IdbClient) -> None:
        self.logger.warning(f"'push' is deprecated, please use 'file push' instead")
        return await FSPushCommand().run_with_bundle(
            bundle_id=args.bundle_id, args=args, client=client
        )


class DeprecatedPullCommand(TargetCommand):
    @property
    def description(self) -> str:
        return "Copy a file inside an application's container"

    @property
    def name(self) -> str:
        return "pull"

    def add_parser_arguments(self, parser: ArgumentParser) -> None:
        parser.add_argument(
            "bundle_id", help="Bundle id of the app to contain these files", type=str
        )
        parser.add_argument("src", help="Relativer Container source path", type=str)
        parser.add_argument("dst", help="Local destination path", type=str)
        super().add_parser_arguments(parser)

    async def run_with_client(self, args: Namespace, client: IdbClient) -> None:
        self.logger.warning(f"'pull' is deprecated, please use 'file pull' instead")
        return await FSPullCommand().run_with_bundle(
            bundle_id=args.bundle_id, args=args, client=client
        )



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

from argparse import REMAINDER, ArgumentParser, Namespace

from idb.cli.commands.base import TargetCommand
from idb.common.misc import get_env_with_idb_prefix
from idb.common.signal import signal_handler_event
from idb.common.types import IdbClient


class LaunchCommand(TargetCommand):
    @property
    def description(self) -> str:
        return (
            "Launch an application. Any environment variables of the form IDB_X\n"
            " will be passed through with the IDB_ prefix removed."
        )

    @property
    def name(self) -> str:
        return "launch"

    def add_parser_arguments(self, parser: ArgumentParser) -> None:
        parser.add_argument(
            "bundle_id", help="Bundle id of the app to launch", type=str
        )
        parser.add_argument(
            "app_arguments",
            help="Arguments to start the app with",
            default=[],
            nargs=REMAINDER,
        )
        parser.add_argument(
            "-f",
            "--foreground-if-running",
            help="If the app is already running foreground that process",
            action="store_true",
        )
        parser.add_argument(
            "-w",
            "--wait-for",
            help="Wait for the process to exit, tailing all output from the app",
            action="store_true",
        )
        super().add_parser_arguments(parser)

    async def run_with_client(self, args: Namespace, client: IdbClient) -> None:
        await client.launch(
            bundle_id=args.bundle_id,
            args=args.app_arguments,
            env=get_env_with_idb_prefix(),
            foreground_if_running=args.foreground_if_running,
            stop=signal_handler_event("launch") if args.wait_for else None,
        )



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.

import asyncio
import importlib
from argparse import ArgumentParser
from logging import Logger
from types import ModuleType
from typing import Dict, List

from idb.common.types import LoggingMetadata


def package_exists(package_name: str) -> bool:
    try:
        return importlib.util.find_spec(package_name) is not None
    except Exception:
        return False


PLUGIN_PACKAGE_NAMES = ["idb.fb.plugin"]
PLUGINS: List[ModuleType] = [
    importlib.import_module(package.name)
    for package in [
        importlib.util.find_spec(package_name)
        for package_name in PLUGIN_PACKAGE_NAMES
        if package_exists(package_name)
    ]
    if package is not None
]


def on_launch(logger: Logger) -> None:
    for plugin in PLUGINS:
        if not hasattr(plugin, "on_launch"):
            continue
        plugin.on_launch(logger)  # pyre-ignore


async def on_close(logger: Logger) -> None:
    await asyncio.gather(
        *[
            plugin.on_close(logger)  # pyre-ignore
            for plugin in PLUGINS
            if hasattr(plugin, "on_close")
        ]
    )


async def before_invocation(name: str, metadata: LoggingMetadata) -> None:
    await asyncio.gather(
        *[
            plugin.before_invocation(name=name, metadata=metadata)  # pyre-ignore
            for plugin in PLUGINS
            if hasattr(plugin, "before_invocation")
        ]
    )


async def after_invocation(name: str, duration: int, metadata: LoggingMetadata) -> None:
    await asyncio.gather(
        *[
            plugin.after_invocation(  # pyre-ignore
                name=name, duration=duration, metadata=metadata
            )
            for plugin in PLUGINS
            if hasattr(plugin, "after_invocation")
        ]
    )


async def failed_invocation(
    name: str, duration: int, exception: Exception, metadata: LoggingMetadata
) -> None:
    await asyncio.gather(
        *[
            plugin.failed_invocation(  # pyre-ignore
                name=name, duration=duration, exception=exception, metadata=metadata
            )
            for plugin in PLUGINS
            if hasattr(plugin, "failed_invocation")
        ]
    )


def on_connecting_parser(parser: ArgumentParser, logger: Logger) -> None:
    for plugin in PLUGINS:
        plugin_parser = getattr(plugin, "on_connecting_parser", None)
        if parser is None:
            continue
        plugin_parser(parser=parser, logger=logger)


def resolve_metadata(logger: Logger) -> LoggingMetadata:
    metadata = {}
    for plugin in PLUGINS:
        plugin_resolver = getattr(plugin, "resolve_metadata", None)
        if not plugin_resolver:
            continue
        resolved = plugin_resolver(logger=logger)
        metadata.update(resolved)
    return metadata


def append_companion_metadata(
    logger: Logger, metadata: Dict[str, str]
) -> Dict[str, str]:
    for plugin in PLUGINS:
        method = getattr(plugin, "append_companion_metadata", None)
        if not method:
            continue
        metadata = method(logger=logger, metadata=metadata)
    return metadata
